#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.Functions.calendar import get_calendar_day_items
from homevee.Functions.weather import get_weather
from homevee.cronjobs import FixedTimeCronjob
from homevee.utils.database import get_database_con
from homevee.utils.firebase_utils import send_notification_to_users


class OverviewNotificationCronjob(FixedTimeCronjob):
    def __init__(self):
        super(OverviewNotificationCronjob, self).__init__(task_name="OverviewNotificationCronjob")

    def task_to_do(self, *args):
        self.send_overview_notifications()

    def get_seconds_to_wait(self, execution_time=None):
        #wait until midnight + 5 minutes
        t = datetime.datetime.today()

        seconds_to_wait = (24-t.hour)*60*60 - ((t.minute * 60) - t.second) + 5*60 #Warten bis 00:05

        return seconds_to_wait

    def send_overview_notifications(self):
        db = get_database_con()

        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM USERDATA")

            users = cur.fetchall()

            for user in users:
                day_overview = ""

                weather = get_weather(1, db)
                weather_text = "Wetter: " + weather[0]['desc'] + ", " + str(weather[0]['temps']['min']) + "°C - " + str(weather[0]['temps']['max']) + "°C"

                today = datetime.datetime.today()
                date_today = today.strftime("%Y-%m-%d")
                calendar = get_calendar_day_items(user['USERNAME'], date_today, db)
                calendar_items = calendar['calendar_entries']
                if len(calendar_items) == 0:
                    calendar_text = "Keine Termine"
                else:
                    if len(calendar_items) > 1:
                        calendar_text = str(len(calendar_items)) + " Termine: "
                        for i in range(0, len(calendar_items)):
                            if i == 0:
                                calendar_text += calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                            else:
                                calendar_text += ", " + calendar_items[i]['name'] + " um " + calendar_items[i]['start'] + " Uhr"
                    else:
                        calendar_text = "Ein Termin: "+calendar_items[0]['name']+" um "+calendar_items[0]['start']+" Uhr"

                day_overview = weather_text+"\n"+calendar_text

                send_notification_to_users([user['USERNAME']], "Deine Tagesübersicht", day_overview, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-

def get_toggle_devices(username, room, db):
    devices = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM URL_TOGGLE WHERE LOCATION = :location",
                    {'location': room})

        for toggle in cur.fetchall():
            devices.append({'name': toggle['NAME'], 'id': toggle['ID'], 'type': 'URL-Toggle',
                'icon': toggle['ICON']})

    return {'toggles': devices}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI import zwave
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *
from homevee.utils.device_types import *

def get_dimmers(username, room, db):
    if not has_permission(username, room, db):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    dimmer = []

    with db:
        cur = db.cursor()

        #Z-Wave Dimmer
        cur.execute("SELECT * FROM ZWAVE_DIMMER WHERE LOCATION == :room", {'room': room})
        for item in cur.fetchall():
            dimmer_item = {'device': item['ID'], 'value': item['VALUE'], 'icon': item['ICON'],
                         'name': item['NAME'], 'type': ZWAVE_DIMMER}
            dimmer.append(dimmer_item)

        cur.close()

        return {'dimmer': dimmer}

def set_dimmer(username, type, id, value, db):
    if type == ZWAVE_DIMMER:
        return set_zwave_dimmer(username, id, value, db)
    else:
        raise ValueError("Type does not exist")

def set_zwave_dimmer(username, id, value, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM ZWAVE_DIMMER WHERE ID == :device",
                    {'device': id})

        data = cur.fetchone()

        if not has_permission(username, data['LOCATION'], db):
            return {'result': 'nopermission'}

        device_id = data['ID']

        result = zwave.device_control.set_multistate_device(device_id, value, db)

        if result['code'] == 200:
            cur.execute("UPDATE ZWAVE_DIMMER SET VALUE = :value WHERE ID = :id",
                        {'value': value, 'id': id})

            return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Functions.room_data import get_room_data
from homevee.Helper.helper_functions import has_permission
from homevee.Manager.scenes import get_scenes
from homevee.items.Room import Room
from homevee.items.Status import *


def add_edit_room(username, room_name, room_key, icon, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    room = Room.load_from_db(room_key)
    room.name = room_name
    room.icon = icon

    try:
        if(room.save_to_db(db)):
            return Status(type=STATUS_OK).get_dict()
    except:
        return Status(type=STATUS_ERROR).get_dict()

def delete_room(user, room_key, db):
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    room = Room.load_from_db(room_key, db)

    roomdata = room.get_room_data(db)

    roomdata = get_room_data(user, room_key, db)['roomdata']

    if len(roomdata) == 1:
        if roomdata[0]['type'] == "scenes":
            scenes = get_scenes(user, room_key, db)

            for scene in scenes['scenes']:
                if scene['room'] == room_key:
                    return "roomhasitems"
    elif len(roomdata) > 1:
        return "roomhasitems"

    try:
        if(room.delete(db)):
            return Status(type=STATUS_OK).get_dict()
    except:
        return Status(type=STATUS_ERROR).get_dict()

def move_items_and_delete_old_room(username, old_room, new_room, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    params = {'oldroom': old_room, 'newroom': new_room}

    query_array = []

    #Funksteckdosen
    query_array.append("UPDATE 'funksteckdosen' SET ROOM = :newroom WHERE ROOM == :oldroom;")

    #Z-Wave
    query_array.append("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :newroom WHERE RAUM == :oldroom;")

    #DIY
    query_array.append("UPDATE 'DIY_DEVICES' SET ROOM = :newroom WHERE ROOM == :oldroom;")
    query_array.append("UPDATE 'DIY_SENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'DIY_REEDSENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'DIY_SWITCHES' SET RAUM = :newroom WHERE RAUM == :oldroom;")

    #Szenen
    query_array.append("UPDATE 'SCENES' SET ROOM = :newroom WHERE ROOM == :oldroom;")

    with db:
        cur = db.cursor()
        for query in query_array:
            cur.execute(query, params)

    return delete_room(username, old_room, db)

def delete_room_with_items(username, room_key, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    params = {'oldroom': room_key}

    query_array = []

    # Funksteckdosen
    query_array.append("DELETE FROM 'funksteckdosen' WHERE ROOM == :oldroom;")

    # Z-Wave
    query_array.append("DELETE FROM 'ZWAVE_SENSOREN' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'ZWAVE_THERMOSTATS' WHERE RAUM == :oldroom;")

    # DIY
    query_array.append("DELETE FROM 'DIY_DEVICES' WHERE ROOM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_REEDSENSORS' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_SENSORS' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_SWITCHES' WHERE RAUM == :oldroom;")

    # Szenen
    query_array.append("DELETE FROM 'SCENES' WHERE ROOM == :oldroom;")

    with db:
        cur = db.cursor()
        for query in query_array:
            cur.execute(query, params)

    return delete_room(username, room_key, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import datetime

'''Gibt das Wetter für die angegebene Anzahl an Tagen zurück'''
def get_weather(daycount, db):
    if daycount > 16:
        daycount = 16

    with db:
        cur = db.cursor()

        cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'WEATHER_CACHE'")

        data = cur.fetchone()

        weather_data = json.loads(data['VALUE'])

        days = weather_data['list']

        output = []

        for i in range(0, min(len(days), daycount)):
            day = days[i]

            relative_days = ["Heute", "Morgen", "Übermorgen"]
            day_names = ["Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"]

            if i > 2:
                date = datetime.datetime.fromtimestamp(day['dt'])

                week_day_number = date.weekday()

                date = day_names[week_day_number]+", "+date.strftime('%d.%m.')
            else:
                date = relative_days[i]

            city_name = weather_data['city']['name']

            icon_link = "http://openweathermap.org/img/w/"+day['weather'][0]['icon']+".png"

            weather_desc = day['weather'][0]['description']
            icon = day['weather'][0]['icon']

            pressure = None
            if "pressure" in day:
                pressure = day['pressure']

            humidity = None
            if "humidity" in day:
                humidity = day['humidity']

            wind_speed = None
            if "speed" in day:
                wind_speed = float(day['speed'])*3.6

            wind_direction = None
            if "deg" in day:
                wind_direction = day['deg']

            clouds = None
            if "clouds" in day:
                clouds = day['clouds']

            rain = None
            if "rain" in day:
                rain = day['rain']

            snow = None
            if "snow" in day:
                snow = day['snow']

            day_item = {'city':city_name, 'date':date, 'temps':day['temp'], 'icon':icon_link, 'iconid':icon,
                        'desc':weather_desc, 'pressure':pressure, 'humidity':humidity, 'windspeed':wind_speed,
                        'winddirection': wind_direction, 'clouds':clouds, 'rain':rain, 'snow':snow}

            output.append(day_item)

        cur.close()

        return output

'''Setzt die Orts-ID zur Wetter-Abfrage'''
def set_weather_city_id(request, db):
    return

'''Gibt alle Orte zur Wetterabfrage zurück'''
def get_weather_city_list(db):
    try:
        with db:
            cur = db.cursor()
            cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'WEATHER_CITY_CACHE'")
            cities = cur.fetchone()['VALUE']

            cur.close()

            return cities
    except:
        return None


#!/usr/bin/python
import json

from pymax.cube import Cube

from homevee.Helper import Logger
from homevee.items.Status import *


def get_devices(cubeip):
    data = []

    with Cube(cubeip) as cube:
        for room in cube.rooms:
            for device in room.devices:
                devicedata = {}

                devicedata['room'] = room.name
                devicedata['title'] = device.name
                # devicedata['temp'] = device.temperature
                devicedata['id'] = device.serial

                data.append(devicedata)

    return json.dumps(data)


def get_device_data(cubeip, deviceid):
    data = {}

    with Cube(cubeip) as cube:
        for room in cube.rooms:
            for device in room.devices:
                if deviceid == device.serial:
                    data['room'] = room.room_id
                    data['addr'] = room.rf_address

                    return data


def get_rooms(cubeip):
    data = []

    with Cube(cubeip) as cube:
        for room in cube.rooms:
            roomdata = {}
            roomdata['name'] = room.name
            Logger.log(room.rf_address)
            # roomdata['address'] = room.rf_address
            roomdata['id'] = room.room_id
            data.append(roomdata)

    return json.dumps(data)


#Sets the temperature
def set_temperature(cubeip, deviceid, temp):
    data = get_device_data(cubeip, deviceid)

    # temperatur berechnen
    tempVal = temp

    with Cube(cubeip) as cube:
        result = cube.set_mode_manual(data['room'], data['addr'], tempVal)

        update_temp(deviceid, temp)
        return Status(type=STATUS_OK).get_dict()


def update_temp(deviceid, temp, db):
    with db:
        cur = db.cursor()
        cur.execute("UPDATE MAX_THERMOSTATS SET LAST_TEMP = " + temp + " WHERE ID == '" + deviceid + "';")
        cur.close()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os

END_OF_MESSAGE = "[END_OF_MESSAGE]"

HOMEVEE_DIR = os.path.dirname(os.path.abspath(__file__))

#DATA_DIR = os.path.join(HOMEVEE_DIR, 'data')
DATA_DIR = os.path.abspath(os.path.join(os.sep, 'homevee_data', 'data'))

#make data dir if not exists
if not os.path.isdir(DATA_DIR):
    os.makedirs(DATA_DIR)

SSL_FULLCHAIN = os.path.join(DATA_DIR, 'fullchain.pem')
SSL_CERT = os.path.join(DATA_DIR, 'cert.pem')

LOCAL_SSL_PRIVKEY = os.path.join(DATA_DIR, 'local_privkey.pem')
LOCAL_SSL_CERT = os.path.join(DATA_DIR, 'local_cert.pem')

HOMEVEE_VERSION_NUMBER = "0.1.1.16"


#!/usr/bin/python
# -*- coding: utf-8 -*-

def handle_mqtt_sensor(device_id, value, db):
	with db:
		cur = db.cursor()

		cur.execute("SELECT * FROM MQTT_SENSORS WHERE ID = :id", {'id': device_id})
		item = cur.fetchone()

		#if(item['LAST_VALUE'] != value):
			#trigger automation

		cur.execute("UPDATE MQTT_SENSORS SET LAST_VALUE = :val WHERE ID = :id",
		{'val': value, 'id': device_id})

	#check if device is used in automation rules



#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.Functions.tv_data import get_tv_plan
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_GET_TV_SCHEDULE

class VoiceGetTvScheduleModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_GET_TV_SCHEDULE

    def get_label(self):
        return "gettvschedule"

    def run_command(self, username, text, context, db):
        return self.get_tv(username, text, context, db)

    def get_tv(self, username, text, context, db):
        return {'msg_speech':'TV-Programm', 'msg_text':'TV-Programm'}


import json
import os
import urllib
from _thread import start_new_thread

import pip
from packaging import version

from homevee.items.Status import *
from homevee.Helper.helper_functions import has_permission
from homevee.utils import constants, firebase_utils, database


def get_homevee_update_version():
    installed_version = constants.HOMEVEE_VERSION_NUMBER

    newest_version = get_newest_version()

    if(newest_version is None):
        return False

    if(version.parse(newest_version) > version.parse(installed_version)):
        return newest_version
    else:
        return None

def get_newest_version():
    url = "https://pypi.org/pypi/Homevee/json"

    try:
        response = urllib.request.urlopen(url).read()
        response = response.decode('utf-8')
        response_json = json.loads(response)
        version = response_json['info']['version']

        return version
    except:
        return None

def check_for_updates():
    new_version = get_homevee_update_version()

    return {
        'updates':{
            'current_version': constants.HOMEVEE_VERSION_NUMBER,
            'new_version': new_version,
            'update_available': (new_version is not None),
            'changelog': "Changelog blabla..." #TODO add changelog or link to actual changelog
        }
    }

'''
Updates the Homevee PIP-Package
Returns true if update was successful,
returns false if there was an error
'''
def do_homevee_update(username, db):
    if(not has_permission(username, "admin", db)):
        return {'error': "nopermission"}

    start_new_thread(update_thread, ())

    return Status(type=STATUS_OK).get_dict()

def update_thread():
    new_version = get_homevee_update_version()

    try:
        pip.main(["install", "--upgrade", "Homevee"])
    except:
        return False

    # TODO texte lokalisieren
    title = "Update"
    body = "Update auf Version " + new_version

    # Send notification to admin
    firebase_utils.send_notification_to_admin(title, body, database.get_database_con())

    # Reboot the system after the update
    os.system('reboot')


#!/usr/bin/python
# -*- coding: utf-8 -*-

import codecs
import json
import random
import time
import urllib.error
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request
import urllib.request

from homevee.Helper import Logger

CONTEXT_VALIDITY = 60

SMART_API_PATH = "https://api.homevee.de/api.php"

def contains_pattern(patterns, text):
    for pattern in patterns:
        #print pattern
        if is_in_order(pattern, text):
            return True
    return False

def is_in_order(words, text):
    indices = []

    word_parts = text.split(" ")

    for word in words:
        found = False
        if isinstance(word, list):
            for word_item in word:
                if found:
                    break
                #print word_item

                if(word_item in word_parts):
                    if(len(indices) > 0):
                        max_index = indices[len(indices)-1]
                        indices.append(word_parts.index(word_item, max_index+1))
                    else:
                        indices.append(word_parts.index(word_item))
                    #print word_item+" found"
                    found = True

            if not found:
                return False
        else:
            #print word

            if (word in word_parts):
                if(len(indices) > 0):
                    max_index = indices[len(indices)-1]
                    indices.append(word_parts.index(word, max_index+1))
                else:
                    indices.append(word_parts.index(word))
                #print word+" found"
            else:
                return False

    last_value = -1
    for index in indices:
        if index is -1 or index <= last_value:
            return False
        last_value = index
    return True

def is_in_order_backup(words, text):
    indices = []

    for word in words:
        found = False
        if isinstance(word, list):
            for word_item in word:
                if found:
                    break
                #print word_item
                position = text.find(word_item)
                if position is not -1:
                    indices.append(position)
                    found = True
            if not found:
                return False
        else:
            #print word
            position = text.find(word)
            if position is not -1:
                indices.append(position)
            else:
                return False

    last_value = -1
    for index in indices:
        if index is -1 or index <= last_value:
            return False
        last_value = index
    return True

def get_no_permission_text():
    return random.choice(["Du hast nicht die nötigen Berechtigungen für diese Aktion.",
		"Dazu fehlen dir die Berechtigungen.",
		"Das darfst du nicht."])

def classify_voice_command(request):
    url = "https://assistant.homevee.de:444/"
    data = urllib.request.urlopen(url + urllib.parse.quote(codecs.encode(request, 'utf-8'))).read()
    return json.loads(data)

def generate_string(answer_data):
    if not isinstance(answer_data, list):
        return str(answer_data)

    chosen_answer = random.choice(answer_data)

    output = ""

    if isinstance(chosen_answer, list):
        for part in chosen_answer:
            output = output + generate_string(part)
    else:
        return str(chosen_answer)
    return str(output)

def set_context(username, context_key, context_data, db):

    Logger.log(context_data)

    context_data = json.dumps(context_data)

    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                    {'user': username})

        cur.execute("INSERT INTO VOICE_COMMAND_USER (USERNAME, CONTEXT_TIME, CONTEXT_KEY, CONTEXT_DATA) VALUES (:username, :time, :key, :data)",
                    {'username': username, 'time': int(time.time()), 'key': context_key, 'data': context_data})

def get_context(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                    {'user': username})

        data = cur.fetchone()

        if data:
            context_data = data

            if(int(time.time()) - int(context_data['CONTEXT_TIME'])) <= CONTEXT_VALIDITY:
                cur.execute("DELETE FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                            {'user': username})

                return context_data
            return None
        else:
            return None

def get_okay():
    try:
        url = SMART_API_PATH + "/?action=okay"
        Logger.log(url)

        response = urllib.request.urlopen(url)
        data = response.read()

        data = data.decode('utf-8')

        if data is not None:
            return data
    except urllib.error.HTTPError as e:
        Logger.log(e)
        return ['Sehr gern', 'Ok', 'Alles klar']

def replace_voice_commands(text, username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM VOICE_COMMAND_REPLACE WHERE USERNAME = :user",
                    {'user': username})

        for item in cur.fetchall():
            text = text.replace(item['TEXT'], item['REPLACE_WITH'])

        return text


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *


def get_voice_replace_items(username, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    replace = []

    with db:
        cur = db.cursor()
        cur.execute("SELECT DISTINCT REPLACE_WITH FROM VOICE_COMMAND_REPLACE WHERE USERNAME = :user",
                    {'user': username})

        for item in cur.fetchall():
            cur.execute("SELECT TEXT FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH == :item AND USERNAME = :user",
                {'user': username, 'item': item['REPLACE_WITH']})

            replacements = []

            for replacement in cur.fetchall():
                replacements.append(replacement['TEXT'])

            replace.append({'replacewith': item['REPLACE_WITH'], 'replacearray': replacements})

        cur.close()

        return {'replacedata': replace}

def add_edit_voice_replace_item(username, replacewith, replaceitems, db):
    with db:
        cur = db.cursor()

        replaceitems = json.loads(replaceitems)

        cur.execute("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                    {'replacewith': replacewith, 'user': username})

        for item in replaceitems:
            cur.execute("INSERT INTO VOICE_COMMAND_REPLACE (USERNAME, REPLACE_WITH, TEXT) VALUES (:user, :replacewith, :text)",
                        {'user': username, 'replacewith': replacewith, 'text': item})

    return Status(type=STATUS_OK).get_dict()

def delete_voice_replace_item(username, replacewith, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                    {'replacewith': replacewith, 'user': username})

    return Status(type=STATUS_OK).get_dict()


from homevee.VoiceAssistant.Modules.CalendarModule.add_calendar_module import VoiceAddCalendarModule
from homevee.VoiceAssistant.Modules.CalendarModule.get_calendar_module import VoiceGetCalendarModule
from homevee.VoiceAssistant.Modules.DeviceControlModule.get_sensor_data_module import VoiceDeviceGetSensorDataModule
from homevee.VoiceAssistant.Modules.DeviceControlModule.rgb_control_module import VoiceRgbDeviceControlModule
from homevee.VoiceAssistant.Modules.DeviceControlModule.set_modes_module import VoiceDeviceSetModesModule
from homevee.VoiceAssistant.Modules.ShoppingListModule.add_to_shopping_list_module import \
    VoiceAddToShoppingListModule
from homevee.VoiceAssistant.Modules.ShoppingListModule.get_shopping_list_module import VoiceGetShoppingListModule
from homevee.VoiceAssistant.Modules.add_nutrition_data_module import VoiceAddNutritionDataModule
from homevee.VoiceAssistant.Modules.calculator_module import VoiceCalculatorModule
from homevee.VoiceAssistant.Modules.conversation_module import VoiceConversationModule
from homevee.VoiceAssistant.Modules.get_activities_module import VoiceGetActivitiesModule
from homevee.VoiceAssistant.Modules.get_jokes_module import VoiceGetJokesModule
from homevee.VoiceAssistant.Modules.get_nutrition_diary_module import VoiceGetNutritionDiaryModule
from homevee.VoiceAssistant.Modules.get_nutrition_info_module import VoiceGetNutritionInfoModule
from homevee.VoiceAssistant.Modules.get_recipes_module import VoiceGetRecipesModule
from homevee.VoiceAssistant.Modules.get_summary_module import VoiceGetSummaryModule
from homevee.VoiceAssistant.Modules.get_tv_schedule_module import VoiceGetTvScheduleModule
from homevee.VoiceAssistant.Modules.get_tv_tipps_module import VoiceGetTvTippsModule
from homevee.VoiceAssistant.Modules.get_weather_module import VoiceGetWeatherModule
from homevee.VoiceAssistant.Modules.get_weekday_module import VoiceGetWeekdayModule
from homevee.VoiceAssistant.Modules.get_wikipedia_definition import VoiceGetWikipediaDefinitionModule
from homevee.VoiceAssistant.Modules.movie_api_module import VoiceMovieApiModule
from homevee.VoiceAssistant.Modules.places_api_module import VoicePlacesApiModule


def get_voice_modules():
    voice_modules = [
        VoiceAddCalendarModule(priority=1),
        VoiceGetCalendarModule(priority=1),
        VoiceAddNutritionDataModule(priority=1),
        VoiceGetActivitiesModule(priority=1),
        VoiceGetNutritionInfoModule(priority=1),
        VoiceGetRecipesModule(priority=1),
        VoiceDeviceGetSensorDataModule(priority=1),
        VoiceGetSummaryModule(priority=1),
        VoiceGetTvScheduleModule(priority=1),
        VoiceGetTvTippsModule(priority=1),
        VoiceGetWeatherModule(priority=1),
        VoiceGetWeekdayModule(priority=1),
        VoiceGetWikipediaDefinitionModule(priority=1),
        VoiceGetJokesModule(priority=1),
        VoiceMovieApiModule(priority=1),
        VoiceGetNutritionDiaryModule(priority=1),
        VoicePlacesApiModule(priority=1),
        VoiceRgbDeviceControlModule(priority=1),
        VoiceDeviceSetModesModule(priority=1),
        VoiceAddToShoppingListModule(priority=1),
        VoiceGetShoppingListModule(priority=1),
        VoiceCalculatorModule(priority=1),
        VoiceConversationModule(priority=0),
        #VoiceModule(priority=1),
    ]

    #voice_modules = sorted(voice_modules, key=lambda x: x.get_priority(), reverse=True)

    return voice_modules


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.get_devices import get_device_value
from homevee.exceptions import NoSuchTypeException
from homevee.items.Device.Switch.Funksteckdose import Funksteckdose
from homevee.items.Room import Room
from homevee.items.Status import *
from homevee.utils.device_types import *

def get_mode(type, id, db):
    if type == FUNKSTECKDOSE:
        item = Funksteckdose.load_from_db(id)
    else:
        raise NoSuchTypeException("type "+type+" does not exist")

    return item.mode

def get_modes(user, room, type, id, db):
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    room = Room.load_from_db(room)

    if id is None or id == "":
        modi = []

        devices = {}

        #Funksteckdose
        devices[FUNKSTECKDOSE] = Funksteckdose.get_all(room, db)

        for device_type in devices:
            for device in devices[device_type]:
                item = device.get_dict()

                modi.append(item)

        return {'modi': modi}

    else:
        return get_mode(type, id, db)


    if id == "":
        modi = []

        with db:
            cur = db.cursor()

            #Funksteckdosen laden
            cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE ROOM == :room", {'room': room})
            for item in cur.fetchall():
                mode_item = {'device': item['DEVICE'], 'mode': item['ZUSTAND'], 'icon': item['ICON'],
                    'name': item['NAME'], 'type': FUNKSTECKDOSE}
                modi.append(mode_item)

            #Z-Wave Schalter
            cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE LOCATION == :room", {'room': room})
            for item in cur.fetchall():
                mode_item = {'device': item['ID'], 'mode': get_modes(username, room, "Z-Wave Schalter", item['ID'], db), 'icon': item['ICON'],
                             'name': item['NAME'], 'type': ZWAVE_SWITCH}
                modi.append(mode_item)

            cur.close()

            return {'modi': modi}
    else:
        with db:
            cur = db.cursor()

            if type == FUNKSTECKDOSE:
                cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :device", {'device': id})
                return cur.fetchone()['ZUSTAND']
            elif type == ZWAVE_SWITCH:
                '''gateway_data = get_gateway("Z-Wave", db)

                port = ""

                if gateway_data['PORT'] is not None and gateway_data['PORT'] != "":
                    port = ":"+gateway_data['IP'];

                link = "http://"+gateway_data['IP']+port+"/ZAutomation/api/v1/devices/"+id

                result = "{}"

                array = json.loads(result)

                if "code" not in array or array['code'] != "200":
                    return "false"

                return str(array['data']['metrics']['level'] == "on")'''
                return get_device_value(ZWAVE_SWITCH, id, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Functions.condition_actions.actions import run_actions
from homevee.items.Status import *


def get_voice_commands(db):

    rules = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM CUSTOM_VOICE_COMMANDS")

        for item in cur.fetchall():
            commands = get_command_data(item['ID'], db)
            responses = get_response_data(item['ID'], db)

            rules.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                          'command_data': commands, 'response_data': responses})

    return {'rules': rules}

def get_command_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

        items = cur.fetchall()

        data = []

        for item in items:
            data.append(item['COMMAND'])

        return data

def get_response_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

        items = cur.fetchall()

        data = []

        for item in items:
            data.append(item['RESPONSE'])

        return data

def add_edit_voice_command(username, id, name, command_data, response_data, action_data, db):
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if(add_new):
            cur.execute("INSERT INTO CUSTOM_VOICE_COMMANDS (NAME, ACTION_DATA) VALUES (:name, :actions)",
                        {'name': name, 'actions': action_data})

            id = cur.lastrowid

            command_data = json.loads(command_data)
            add_command_data(command_data, id, db)

            response_data = json.loads(response_data)
            add_response_data(response_data, id, db)
            return Status(type=STATUS_OK).get_dict()

        else:
            cur.execute("UPDATE AUTOMATION_DATA SET NAME = :name, ACTION_DATA = :actions WHERE ID = :id",
                {'name': name, 'actions': action_data, 'id': id})

            cur.execute("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})
            cur.execute("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})


            command_data = json.loads(command_data)
            add_command_data(command_data, id, db)

            response_data = json.loads(response_data)
            add_response_data(response_data, id, db)

            return Status(type=STATUS_OK).get_dict()

def delete_voice_command(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM CUSTOM_VOICE_COMMANDS WHERE ID = :id", {'id': id})

        cur.execute("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

        cur.execute("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

        return Status(type=STATUS_OK).get_dict()

def add_command_data(commands, id, db):
    with db:
        cur = db.cursor()
        for command in commands:
            param_array = {'id': id, 'command': command.lower()}

            cur.execute(
                "INSERT INTO CUSTOM_VOICE_COMMAND_SENTENCES (COMMAND_ID, COMMAND) VALUES (:id, :command)",
                param_array)

def add_response_data(responses, id, db):
    with db:
        cur = db.cursor()
        for response in responses:
            param_array = {'id': id, 'response': response}

            cur.execute(
                "INSERT INTO CUSTOM_VOICE_COMMAND_RESPONSES (COMMAND_ID, RESPONSE) VALUES (:id, :response)",
                param_array)

def run_custom_voice_commands(text, username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES, CUSTOM_VOICE_COMMANDS WHERE ID = COMMAND_ID AND COMMAND = :command",
                    {'command': text})

        result = cur.fetchone()

        if(result is None):
            return None

        id = result['ID']

        action_data = result['ACTION_DATA']

        action_data = json.loads(action_data)

        #run actions
        run_actions(action_data, db)

        cur.execute("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id ORDER BY RANDOM()",
                    {'id': id})

        return cur.fetchone()['RESPONSE']


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.DeviceAPI import blind_control
from homevee.DeviceAPI.heating import heating_control
from homevee.DeviceAPI.rgb_control import rgb_control
from homevee.DeviceAPI.set_modes import set_modes
from homevee.DeviceAPI.wake_on_lan import wake_on_lan, wake_xbox_on_lan
from homevee.utils.device_types import *
from homevee.utils.firebase_utils import send_notification_to_users
from homevee.items.Status import *


def run_scene(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM SCENES WHERE ID = :id",
                    {'id': id})

        result = cur.fetchone()

        if (result is None):
            return None

        id = result['ID']

        action_data = result['ACTION_DATA']

        action_data = json.loads(action_data)

        # run actions
        run_actions(action_data, db)

        return Status(type=STATUS_OK).get_dict().get_dict()


def run_actions(action_data, db):
    for action in action_data:
        if action['type'] == "push_notification":
            msg = action['message']
            users = action['users']
            send_notification_to_users(users, "Homevee", msg, db)
        elif action['type'] == "run_scene":
            scene_id = action['id']
            run_scene(None, scene_id, db)
        elif action['type'] == 'control_device':
            device_type = action['devicetype']
            device_id = action['id']

            if device_type in [FUNKSTECKDOSE, URL_TOGGLE, URL_SWITCH, ZWAVE_SWITCH]:
                set_modes(None, device_type, device_id, action['value'], db, False)
            elif device_type == WAKE_ON_LAN:
                wake_on_lan(None, device_id, db, False)
            elif device_type == XBOX_ONE_WOL:
                wake_xbox_on_lan(None, device_id, db, False)
            elif device_type in [ZWAVE_THERMOSTAT, MAX_THERMOSTAT, RADEMACHER_THERMOSTAT]:
                heating_control(None, device_type, device_id, action['value'], db, False)
            elif device_type in [RADEMACHER_BLIND_CONTROL]:
                blind_control.set_blinds(None, device_type, device_id, action['value'], db, False)
            elif device_type in [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]:
                data = json.loads(action['value'])
                rgb_control(None, device_type, device_id, data['mode'], data['brightness'], data['color'], db, False)





#!/usr/bin/python
# -*- coding: utf-8 -*-

def get_places(username, keyword, lat, lng, db):
    places = []

    return places

def get_place_data(username, place_id, db):
    data = {}

    return data


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
from homevee.items.Status import *


def get_home_budget_data(username, startdate, db):
    data = {}

    with db:
        cur = db.cursor()

        #Find expenditures grouped by date
        cur.execute("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as DATE FROM HOME_BUDGET_DATA WHERE AMOUNT <= 0 AND DATE >= :date GROUP BY DATE ORDER BY DATE ASC",
                    {'date': startdate})
        costs = []

        last_date = None

        for item in cur.fetchall():
            this_date = datetime.datetime.strptime(item['DATE'], "%d.%m.%Y")
            if(last_date is not None):
                difference = (this_date - last_date).days

                for i in range(1, difference):
                    date = last_date + datetime.timedelta(days=i)
                    costs.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

            last_date = this_date

            costs.append({'date': item['DATE'], 'amount': round(item['AMOUNT']*-1, 2)})
        data['costs'] = costs

        #Find amount money left
        cur.execute("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA")
        item = cur.fetchone()
        overall_amount = round(item['AMOUNT'], 2)
        data['overall_amount'] = overall_amount

        #Compute date where money reaches 0
        cur.execute("SELECT AVG(AMOUNT) as AMOUNT FROM (SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE AMOUNT < 0 GROUP BY DATE);")
        item = cur.fetchone()
        average_spending_per_day = item['AMOUNT']*-1

        #compute timeperiod
        cur.execute("SELECT MIN(DATE) as mindate, MAX(DATE) as maxdate, COUNT(*) as numdays FROM (SELECT DISTINCT DATE FROM HOME_BUDGET_DATA);")
        min_max_date = cur.fetchone()
        real_numdays = (datetime.datetime.today()-datetime.datetime.strptime(min_max_date['mindate'], "%Y-%m-%d")).days
        average_spending_per_day = average_spending_per_day * min_max_date['numdays'] / real_numdays
        days_left = overall_amount/average_spending_per_day
        date = datetime.datetime.today() + datetime.timedelta(days=days_left)
        date = datetime.datetime.strftime(date, "%d.%m.%Y")
        data['sufficient_until'] = date

    return {'data': data}

def get_home_budget_data_graph(username, startdate, enddate, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE DATE < :startdate;",
            {'startdate': startdate})

        startvalue = cur.fetchone()['AMOUNT']

        if startvalue is None:
            startvalue = 0

        #Get values
        cur.execute("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE >= :startdate AND DATE <= :enddate GROUP BY DATE ORDER BY DATE ASC",
            {'startdate': startdate, 'enddate': enddate})

        values = []

        last_date = None

        for item in cur.fetchall():
            this_date = datetime.datetime.strptime(item['FORMATTED_DATE'], "%d.%m.%Y")
            if (last_date is not None):
                difference = (this_date - last_date).days

                for i in range(1, difference):
                    date = last_date + datetime.timedelta(days=i)
                    values.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

            last_date = this_date

            values.append({'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2)})

    return {'startvalue': startvalue, 'values': values}


def get_home_budget_data_day_items(username, date, db):
    data = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT *, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE = :date ORDER BY DATE ASC",
                    {'date': date})

        for item in cur.fetchall():
            data.append({'id': item['ID'], 'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2), 'info': item['INFO']})

    return {'data': data}

def add_edit_home_budget_data(username, id, date, info, amount, db):
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if (add_new):
            cur.execute("INSERT INTO HOME_BUDGET_DATA (INFO, AMOUNT, DATE) VALUES (:info, :amount, :date)",
                        {'info': info, 'amount': float(amount), 'date': date})

            return Status(type=STATUS_OK).get_dict().get_dict()

        else:
            cur.execute("UPDATE HOME_BUDGET_DATA SET INFO = :info, AMOUNT = :amount, DATE = :date WHERE ID = :id",
                        {'info': info, 'amount': float(amount), 'date': date, 'id': id})

            return Status(type=STATUS_OK).get_dict().get_dict()

def delete_home_budget_data(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM HOME_BUDGET_DATA WHERE ID = :id", {'id': id})

        return Status(type=STATUS_OK).get_dict().get_dict()


import urllib.request

from homevee.exceptions import *
from homevee.items.Device.Switch import Switch
from homevee.items.Gateway import Gateway, FUNKSTECKDOSEN_CONTROLLER
from homevee.utils import database
from homevee.utils.device_types import FUNKSTECKDOSE


class Funksteckdose(Switch):
    def __init__(self, name, icon, location, home_code, socket_number, id=None, mode=False):
        super(Funksteckdose, self).__init__(name, icon, location, id=id, mode=mode)
        self.home_code = home_code
        self.socket_number = socket_number

    def get_device_type(self):
        return FUNKSTECKDOSE

    def delete(self, db=None):
        try:
            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM FUNKSTECKDOSEN WHERE DEVICE == :id'", {'id': self.id})

                cur.close()
            return True
        except:
            return False

    def save_to_db(self, db=None):
        try:
            if(db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                #insert
                if(self.id is None or self.id == ""):
                    cur.execute("INSERT INTO FUNKSTECKDOSEN (ROOM, NAME, ICON, HAUSCODE, STECKDOSENNNUMMER) "
                                "VALUES (:location, :name, :icon, :home_code, :socket_number)",
                                {'location': self.location, 'name': self.name, 'icon': self.icon,
                                 'home_code': self.home_code, 'socket_number': self.socket_number})
                #update
                else:
                    cur.execute("UPDATE FUNKSTECKDOSEN SET ROOM = :location, NAME = :name, ICON = :icon,"
                                "HAUSCODE = :home_code, STECKDOSENNUMMER = :socket_number, ZUSTAND = :mode"
                                "WHERE DEVICE = :id",
                                {'location': self.location, 'name': self.name, 'icon': self.icon,
                                 'home_code': self.home_code, 'socket_number': self.socket_number,
                                 'mode': self.mode, 'id': self.id})

                cur.close()

                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'home_code': self.home_code,
            'socket_number': self.socket_number,
            'id': self.id,
            'mode': self.mode,
            'location': self.location
        }
        return dict

    def update_mode(self, mode, db=None):
        try:
            gateway = Gateway.load_from_db(FUNKSTECKDOSEN_CONTROLLER)

            url = 'http://' + str(gateway.ip) + '/funksteckdose.php?hauscode=' + str(self.home_code) + \
                  '&steckdosennummer=' + str(self.socket_number) + "&zustand=" + str(mode)
            #Logger.log(url)
            result = urllib.request.urlopen(url)

            return True

        except:
            return False

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return Funksteckdose.load_all_from_db('SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)
            for result in cur.fetchall():
                item = Funksteckdose(result['NAME'], result['ICON'], result['ROOM'], result['HAUSCODE'],
                                     result['STECKDOSENNUMMER'], result['DEVICE'], result['ZUSTAND'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def load_from_db(id, db=None):
        items = Funksteckdose.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find funksteckdosen-id: " + id)
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return Funksteckdose.load_all_from_db('SELECT * FROM FUNKSTECKDOSEN', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            home_code = dict['home_code']
            socket_number = dict['socket_number']
            mode = dict['mode']
            icon = dict['icon']

            funksteckdose = Funksteckdose(name, icon, location, home_code, socket_number, id, mode)

            return funksteckdose

        except:
            raise InvalidParametersException("Invalid parameters for Funksteckdose.create_from_dict()")

    @staticmethod
    def get_all(location=None, db=None):
        devices = []

        all_devices = Funksteckdose.load_all(db)

        if location is None or location == "all":
            # get all devices of all types
            return all_devices

        for device in all_devices:
            if str(device.location) == str(location.id):
                devices.append(device)

        return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re

from homevee.Functions.nutrition_data import get_nutrition_data, add_edit_user_day_nutrition_item, get_user_nutrition_overview
from homevee.Helper import Logger
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.context_keys import CONTEXT_ADD_NUTRITION
from homevee.VoiceAssistant.helper import set_context
from homevee.VoiceAssistant.voice_patterns import *

MAX_PORTION_DEVIATION = 10

class VoiceAddNutritionDataModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_ADD_NUTRITION_ITEM

    def get_label(self):
        return "addnutrition"

    def run_command(self, username, text, context, db):
        return self.voice_add_nutrition_item(username, text, context, db)

    def voice_add_nutrition_item(self, username, text, context, db):
        answer = "add_nutrition_item"

        text = re.sub(r'([\d]+)g', '\1 Gramm', text)

        Logger.log(text)

        words = text.split(" ")

        number_words = ['ein', 'einer', 'eine', 'einem', 'einen']

        amount_words = ['gramm', 'milliliter', 'kilo', 'kilogramm', 'liter', 'stück', 'scheiben', 'scheibe', 'brötchen']

        eat_words = ['esse', 'gegessen', 'essen', 'ess', 'trink', 'trinke', 'trinken', 'getrunken']

        time_words = ['jetzt', 'gerade', 'eben']

        daytime_words = ['morgens', 'morgen', 'mittag', 'mittags', 'abend', 'abends', 'snack', 'snacks', 'zwischendurch']

        is_name = False

        item = None

        amount = None
        amount_type = None

        daytime = None

        current_hour = datetime.datetime.now().hour

        for word in words:
            if(word in time_words):
                daytime = 'snacks'
                if current_hour >= 5 and current_hour < 10:
                    daytime = 'morning'
                elif current_hour >= 11 and current_hour < 13:
                    daytime = 'noon'
                elif current_hour >= 17 and current_hour < 19:
                    daytime = 'evening'

            if(word in daytime_words and daytime is None):
                if word in ['morgens', 'morgen']:
                    daytime = 'morning'
                elif word in ['mittag', 'mittags']:
                    daytime = 'noon'
                elif word in ['abends', 'abend']:
                    daytime = 'evening'
                elif word in ['snack', 'snacks', 'zwischendurch']:
                    daytime = 'snacks'

            if is_name and word in eat_words:
                is_name = False

            if is_name:
                if item is None:
                    item = word
                else:
                    item = item + " " +word

            if word in number_words:
                is_name = True
                amount = 1

            if word in amount_words:
                amount = words[words.index(word)-1]
                amount_type = word
                is_name = True

        Logger.log(str(item)+" => "+str(amount)+" => "+str(amount_type)+" => "+str(daytime))

        nutrition_data = None

        if(context is not None):

            if('query' in context):
                if(context['query'] == 'amount'):
                    for word in words:
                        if word in number_words:
                            amount = 1
                            break

                    if amount is None:
                        numbers_in_text = [int(s) for s in text.split() if s.isdigit()]

                        if (len(numbers_in_text) != 0):
                            amount = numbers_in_text[0]
                elif(context['query'] == 'item'):
                    item = text
                elif(context['query'] == 'daytime'):
                    for word in words:
                        if (word in daytime_words and daytime is None):
                            if word in ['morgens', 'morgen']:
                                daytime = 'morning'
                            elif word in ['mittag', 'mittags']:
                                daytime = 'noon'
                            elif word in ['abends', 'abend']:
                                daytime = 'evening'
                            elif word in ['snack', 'snacks', 'zwischendurch']:
                                daytime = 'snacks'

            if daytime is None and 'daytime' in context and context['daytime'] is not None:
                daytime = context['daytime']

            if amount is None and 'amount' in context and context['amount'] is not None:
                amount = context['amount']

            if amount_type is None and 'amount_type' in context and context['amount_type'] is not None:
                amount_type = context['amount_type']

            if nutrition_data is None and 'nutrition_data' in context and context['nutrition_data'] is not None:
                nutrition_data = context['nutrition_data']
                item = nutrition_data['name']

        #print "Amount: "+str(amount)

        if(item is not None):
            if(nutrition_data is None):
                nutrition_data = get_nutrition_data(username, item, db)

            if(nutrition_data is None or nutrition_data is False):
                answer = "Zum Suchbegriff "+item+" wurden keine Einträge gefunden."
                return {'msg_speech': answer, 'msg_text': answer}

            date = datetime.datetime.today().strftime("%d.%m.%Y")

            if(amount is None):
                answer = "Wie viel " + nutrition_data['portionunit'] + " hast du von " + nutrition_data[
                    'name'] + " verzehrt?"
                context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                           'daytime': daytime, 'query': 'amount'}
                set_context(username, CONTEXT_ADD_NUTRITION, context, db)
            elif(daytime is None):
                answer = "Wann hast du " + nutrition_data['name'] + " verzehrt? Morgens, Mittags, Abends oder zwischendurch?"
                context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                           'daytime': daytime, 'query': 'daytime'}
                set_context(username, CONTEXT_ADD_NUTRITION, context, db)
            else:
                #print "amount: "+str(amount)
                portion_deviation = float(amount) / float(nutrition_data['portionsize'])

                #print portion_deviation

                if(portion_deviation >= MAX_PORTION_DEVIATION or portion_deviation <= 1.0/float(MAX_PORTION_DEVIATION)):
                    #invalid portion size

                    answer = "Wie viel "+nutrition_data['portionunit']+" hast du von "+nutrition_data['name']+" verzehrt?"
                    context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'amount'}
                    set_context(username, CONTEXT_ADD_NUTRITION, context, db)
                else:
                    #anhand der aktuellen uhrzeit die daytime errechnen

                    answer = "Ok, " + str(amount) + " " + nutrition_data['portionunit'] + " "+ nutrition_data['name'] +" wurde deinem Ernährungstagebuch hinzugefügt."

                    add_edit_user_day_nutrition_item(username, None, date, daytime, nutrition_data['name'], amount,
                                                 nutrition_data['portionsize'], nutrition_data['portionunit'],
                                                 nutrition_data['calories'], nutrition_data['fat'], nutrition_data['saturated'],
                                                 nutrition_data['unsaturated'], nutrition_data['carbs'], nutrition_data['sugar'],
                                                 nutrition_data['protein'], db)

                    calories_left = get_user_nutrition_overview(username, db)['data']['nutrition_day_data']['calories_left']

                    if (calories_left >= 0):
                        answer = answer + ' Du hast heute noch ' + str(calories_left) + ' Kalorien übrig.'
                    else:
                        answer = answer + ' Du bist heute ' + str(-1*calories_left) + ' Kalorien über deinem Ziel.'
        else:
            answer = "Was möchtest du deinem Ernährungstagebuch hinzufügen?"
            context = {'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'item'}
            set_context(username, CONTEXT_ADD_NUTRITION, context, db)

        #determine amount

        #determine product

        return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.get_modes import get_modes
from homevee.DeviceAPI.set_modes import set_modes
from homevee.DeviceAPI.wake_on_lan import wake_on_lan
from homevee.Helper import Logger
from homevee.Manager.dashboard import get_device_info
from homevee.utils.device_types import *
from homevee.items.Status import *


def get_rfid_tags(username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM RFID_TAGS")

        tags = []

        for item in cur.fetchall():
            try:
                device_info = get_device_info(username, item['ACTION_TYPE'], item['ACTION_ID'], db)

                Logger.log(item)
                Logger.log(device_info)

                tag = {'name': item['NAME'], 'uuid': item['UUID'], 'actiontype': item['ACTION_TYPE'],
                       'actionid': item['ACTION_ID'], 'roomname': device_info['roomname'], 'actionname': device_info['name']}
                tags.append(tag)
            except:
                continue

        return tags

def add_edit_rfid_tag(username, name, uuid, actionType, actionId, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT COUNT(*) FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

        is_new = cur.fetchone()['COUNT(*)'] is not 1

        param_array = {'name': name, 'uuid': uuid, 'type': actionType, 'id': actionId}

        if is_new:
            cur.execute("INSERT INTO RFID_TAGS (NAME, UUID, ACTION_TYPE, ACTION_ID) VALUES (:name, :uuid, :type, :id)",
                        param_array)
        else:
            cur.execute("UPDATE RFID_TAGS SET NAME = :name, ACTION_TYPE = :type, ACTION_ID = :id WHERE UUID = :uuid",
                        param_array)


    return Status(type=STATUS_OK).get_dict()

def delete_rfid_tag(userane, uuid, db):
    with db:
        cur = db.cursor()

        cur.execue("DELETE FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

    return Status(type=STATUS_OK).get_dict()


def wake_xbob_on_lan(username, id, db):
    pass


def run_rfid_action(username, uuid, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

        result = cur.fetchone()

        if result is None:
            return {'result': 'tagnotfound'}
        else:
            type = result['ACTION_TYPE']
            id = result['ACTION_ID']

            if type in [FUNKSTECKDOSE, ZWAVE_SWITCH, URL_SWITCH, URL_TOGGLE]:
                mode = get_modes(username, None, type, id, db)

                if mode == "1" or mode == 1 or mode == True or mode == "true":
                    mode = 0
                else:
                    mode = 1

                result = set_modes(username, type, id, mode, db)

            elif type == WAKE_ON_LAN:
                result = wake_on_lan(username, id, db)

            elif type == XBOX_ONE_WOL:
                result = wake_xbob_on_lan(username, id, db)

    return Status(type=STATUS_OK).get_dict()


import json
import socket
import ssl
import traceback

# https://github.com/olucurious/PyFCM
from homevee.Helper import Logger
from .constants import END_OF_MESSAGE


def send_notification(registration_ids, message_title, message_body, click_action=None):
    CLOUD_URL = "free.cloud.homevee.de"
    CLOUD_PORT = 7778

    conn = None

    try:
        conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        conn.settimeout(5 * 60)

        # SSL aktivieren
        conn = ssl.wrap_socket(conn)

        # Logger.log("Wrapping SSL")

        conn.connect((CLOUD_URL, CLOUD_PORT))

        # print ("SSL connected")

        # Logger.log("Connection successful!")

        message_data = {
            'title': message_title,
            'msg': message_body,
            'clickaction': click_action
        }

        data_dict = {'registration_ids': registration_ids, 'message_data': json.dumps(message_data)}

        data = json.dumps(data_dict) + END_OF_MESSAGE

        len_send = conn.send(data)

        while 1:
            data = conn.recv(8192)

            if data == "":
                conn.close()
                return

            Logger.log("")
            Logger.log(("Received: " + data))

            data_parts = data.split(END_OF_MESSAGE)
            for data_part in data_parts:
                if data_part is None or data_part == "":
                    continue

                Logger.log("")
                Logger.log(data_part)

                data_dict = json.loads(data_part)

                if 'result' in data_dict:
                    Logger.log(data_dict)

            break

    except Exception as e:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

        if conn is not None:
            conn.close()

def send_notification_to_users(users, message_title, message_body, db, click_action=None):
    registration_ids = get_user_tokens(users, db)

    send_notification(registration_ids, message_title, message_body, click_action)

def send_notification_to_admin(message_title, message_body, db, click_action=None):
    admins = []

    send_notification_to_users(admins, message_title, message_body, db, click_action)

def get_user_tokens(users, db):
        tokens = []

        with db:
            if (len(users) is 0):
                cur = db.cursor()

                cur.execute("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS")

                for item in cur.fetchall():
                    tokens.append(item['TOKEN'])
            else:
                cur = db.cursor()

                for user in users:
                    cur.execute("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user",
                                {'user': user})

                    for item in cur.fetchall():
                        if (item['TOKEN'] is not None):
                            tokens.append(item['TOKEN'])

        #print users
        #print tokens

        return tokens

def delete_user_tokens(username, db):
    with db:
        cur = db.cursor()
        cur.execute("DELETE FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user", {'user': username})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request

from homevee.items.Gateway import *


def get_devices(db):
    gateway = Gateway.load_from_db(RADEMACHER_HOMEPILOT, db)

    devices = []

    url = "http://"+gateway.ip+"/deviceajax.do?devices=1"

    response = urllib.request.urlopen(url).read()

    data = json.loads(response)

    for device in data['devices']:
        item = {'title': device['name'], 'id': device['did'], 'info': device['description']}

        devices.append(item)

    return {'devices': devices}


from homevee.Helper import Logger
from homevee.testing import benchmark
from homevee.testing.test_voice_assistant import run_voice_tests


def run_tests():
    print("running tests...")
    #run_benchmarks
    #benchmark.do_benchmarks()

    Logger.IS_DEBUG = False

    run_voice_tests()

if __name__ == "__main__":
    run_tests()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.context_keys import CONTEXT_CALCULATOR
from homevee.VoiceAssistant.voice_patterns import PATTERN_CALCULATOR

class VoiceCalculatorModule(VoiceModule):
    def calculator(self, username, text, context, db):
        return {'msg_speech':"Taschenrechner", 'msg_text':"Taschenrechner"}

    def get_pattern(self, db):
        return PATTERN_CALCULATOR

    def get_label(self):
        return "calculator"

    def get_context_key(self):
        return CONTEXT_CALCULATOR

    def run_command(self, username, text, context, db):
        return self.calculator(username, text, context, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.wake_on_lan import wake_on_lan
from homevee.Helper import Logger
from homevee.VoiceAssistant import context_keys
from homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule
from homevee.VoiceAssistant.helper import generate_string, set_context, get_okay
from homevee.VoiceAssistant.voice_patterns import GET_PATTERN_WOL
from homevee.utils.device_types import *

class VoiceDeviceWakeOnLanModule(VoiceDeviceControlModule):
    def get_context_key(self):
        return "VOICE_WAKE_ON_LAN"

    def get_pattern(self, db):
        return GET_PATTERN_WOL

    def get_label(self):
        return "wakeonlan"

    def voice_wol(self, username, text, context, db):
        room = self.find_room(text, db)

        if room is not None:
            room_key = room['LOCATION']
        else:
            room_key = None

        device_types = [WAKE_ON_LAN, XBOX_ONE_WOL]

        devices = self.find_devices(text, device_types, room_key, db)

        if len(devices) == 0:
            #Keine Geräte gefunden
            answer_data = [
                [['Dieses ', 'Das genannte '], 'Gerät ', ['existiert nicht.', 'gibt es nicht.', 'wurde noch nicht angelegt.']]
            ]

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}

        words = text.split(" ")

        #Geräte schalten
        for device in devices:
            if device['type'] == XBOX_ONE_WOL:
                Logger.log("")
                #xbox_wake_up(username, device['id'], db)
            else:
                wake_on_lan(username, device['id'], db)

        set_context(username, context_keys.CONTEXT_WOL, {'location': room, 'devices': devices}, db)

        DEVICE_STRING = None
        if len(devices) > 1:
            VERB = "wurden"
            DEVICE_WORD = "Die Geräte "

            for i in range(0, len(devices)):
                if DEVICE_STRING is None:
                    DEVICE_STRING = '\'' + devices[i]['name'] + '\''
                elif i == len(devices) - 1:
                    DEVICE_STRING = DEVICE_STRING + ' und ' + '\'' + devices[i]['name'] + '\''
                else:
                    DEVICE_STRING = DEVICE_STRING + ', ' + '\'' + devices[i]['name'] + '\''
        else:
            DEVICE_WORD = "Das Gerät "
            VERB = "wurde"

            DEVICE_STRING = '\'' + devices[0]['name'] + '\''

        answer_data = [
            [get_okay(), [', ' + username, ''], '.',
             ['', [' ', DEVICE_WORD,  [DEVICE_STRING + ' ', ''], VERB + ' ',
                   ['gestartet', 'eingeschalten', 'eingeschaltet', 'an gemacht', 'hochgefahren'], '.']]]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import http.server
import socketserver

import os

from homevee.Helper import Logger

PORT = 8000

def start_http_server():
    Handler = http.server.SimpleHTTPRequestHandler

    httpd = socketserver.TCPServer(("", PORT), Handler)

    os.chdir(os.path.dirname(os.path.realpath(__file__)))

    Logger.log("HTTP-Server running on port: "+str(8000))

    httpd.serve_forever()


from homevee.exceptions import *
from homevee.items.Device.Sensor import Sensor
from homevee.utils import database
from homevee.utils.device_types import ZWAVE_SENSOR


class ZWaveSensor(Sensor):
    def __init__(self, name, icon, location, save_data, sensor_type, id=None, value=None):
        super(ZWaveSensor, self).__init__(name, icon, location, save_data, sensor_type, id=id, value=value)

    def get_device_type(self):
        return ZWAVE_SENSOR

    def delete(self, db=None):
        try:
            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM ZWAVE_SENSOREN WHERE ID == :id'", {'id': self.id})

                cur.close()
            return True
        except:
            return False

    def save_to_db(self, db=None):
        try:
            if(db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                #insert
                if(self.id is None or self.id == ""):
                    cur.execute("INSERT INTO ZWAVE_SENSOREN (RAUM, SHORTFORM, ICON, SAVE_DATA,"
                                "SENSOR_TYPE, VALUE) VALUES (:location, :name, :icon, :save_data,"
                                ":sensor_type)",
                                {'location': self.location, 'name': self.name, 'icon': self.icon,
                                 'save_data': self.save_data, 'sensor_type': self.sensor_type})
                #update
                else:
                    cur.execute("UPDATE ZWAVE_SENSOREN SET RAUM = :location, SHORTFORM = :name, ICON = :icon,"
                                "SAVE_DATA = :save_data, SENSOR_TYPE = :sensor_type, VALUE = :value"
                                "WHERE ID = :id",
                                {'location': self.location, 'name': self.name, 'icon': self.icon,
                                 'save_data': self.save_data, 'sensor_type': self.sensor_type,
                                 'value': self.value, 'id': self.id})

                cur.close()

                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'value': self.value,
            'id': self.id,
            'location': self.location,
            'save_data': self.save_data,
            'sensor_type': self.sensor_type
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return ZWaveSensor.load_all_from_db('SELECT * FROM ZWAVE_SENSOREN WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)
            for result in cur.fetchall():
                item = ZWaveSensor(result['SHORTFORM'], result['ICON'], result['RAUM'], result['SAVE_DATA'],
                                     result['SENSOR_TYPE'], result['ID'], result['VALUE'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def load_from_db(id, db=None):
        items = ZWaveSensor.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find zwave-sensor-id: " + id)
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return ZWaveSensor.load_all_from_db('SELECT * FROM ZWAVE_SENSOREN', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            value = dict['value']
            icon = dict['icon']
            save_data = dict['save_data']
            sensor_type = dict['sensor_type']

            item = ZWaveSensor(name, icon, location, save_data, sensor_type, id, value)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for ZWaveSensor.create_from_dict()")

    @staticmethod
    def get_all(location=None, db=None):
        devices = []

        all_devices = ZWaveSensor.load_all(db)

        if location is None or location == "all":
            # get all devices of all types
            return all_devices

        for device in all_devices:
            if str(device.location) == str(location.id):
                devices.append(device)

        return devices


import os
import time

from homevee.Functions import gps_data, sensor_data, room_data
from homevee.Functions.system_info import get_system_info
from homevee.Helper import Logger
from homevee.utils.database import get_database_con


# import matplotlib.pyplot as plt

class LineChart():
    def __init__(self, value_map):
        self.value_map = value_map

    def plot(self, save_plot=False, file_name='plot.png'):
        fig, ax = plt.subplots()  # create figure & 1 axis

        xVals = []
        yVals = []

        for x in self.value_map:
            xVals.append(x)
            y =self.value_map[x]
            Logger.log(x, y)
            yVals.append(y)

        ax.scatter(xVals, yVals)

        plt.show()

        if save_plot:
            fig.savefig(os.path.join(os.path.dirname(os.path.realpath(__file__)), file_name))

def run_benchmark(function, args, num, name):
    results = {}

    total_time = 0

    Logger.log("running: "+name)

    for i in range(0, num):
        start_time = time.time()
        function(*args)
        end_time = time.time()

        run_time = float(end_time - start_time)*1000*1000

        results[i+1] = run_time

        total_time += run_time

    LineChart(results).plot(False)

    avg_time = float(total_time)/num

    Logger.log(name+": " + str(avg_time) + " nanoseconds")

    return avg_time

def do_benchmarks():
    Logger.log("running benchmarks...")

    db = get_database_con()

    run_time = run_benchmark(get_system_info, [], 1000, "get_system_info()")

    run_time = run_benchmark(gps_data.get_gps_locations, ["sascha", db], 1000, "get_gps_locations()")

    run_time = run_benchmark(sensor_data.get_sensor_data, ["sascha", "all", "", "", "", db], 1000, "get_sensor_data()")

    run_time = run_benchmark(room_data.get_rooms, ["sascha", db], 1000, "get_rooms()")

if __name__ == "__main__":
    do_benchmarks()


#!/usr/bin/python
# -*- coding: utf-8 -*-

FUNKSTECKDOSE = "Funksteckdose"
MAX_THERMOSTAT = "MAX! Thermostat"
ZWAVE_SENSOR = "Z-Wave Sensor"
ZWAVE_THERMOSTAT = "Z-Wave Thermostat"
ZWAVE_POWER_METER = "Z-Wave Stromzähler"
ZWAVE_DIMMER = "Z-Wave Dimmer"
ZWAVE_SWITCH = "Z-Wave Schalter"
MEDIA_CENTER = "Media Center"
URL_SWITCH = "URL-Schalter"
URL_TOGGLE = "URL-Toggle"
URL_RGB_LIGHT = "URL RGB-Licht"
MQTT_TRIGGER = "MQTT-Trigger"
MQTT_SENSOR = "MQTT Sensor"
IP_CAM = "IP Kamera"
WAKE_ON_LAN = "Wake on Lan Gerät"
XBOX_ONE_WOL = "XBOX One"
PHILIPS_HUE_LIGHT = "Philips Hue Lampe"
RADEMACHER_BLIND_CONTROL = "Rademacher Rolladensteuerung"
RADEMACHER_THERMOSTAT = "Rademacher Thermostat"



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import time

from homevee.items.Status import *
from homevee.utils.firebase_utils import send_notification_to_users


def get_chat_messages(username, time, limit, db):
    messages = []

    with db:
        where_clause = ""
        params = {'limit':limit}

        if int(time) is not -1:
            where_clause = "WHERE TIMESTAMP < :time"
            params['time'] = time

        cur = db.cursor()
        cur.execute("SELECT * FROM CHAT_DATA "+where_clause+" ORDER BY TIMESTAMP DESC LIMIT :limit",
            params)

        for message in cur.fetchall():
            time_string = message['TIMESTAMP']

            #Zeit formattieren

            message_data = {'username': message['USERNAME'], 'data': json.loads(message['DATA']),
                'time': time_string, 'fromotheruser': not(username == message['USERNAME'])}
            messages.append(message_data)

        cur.close()

        return {'messages': messages}

def get_chat_image(username, imageid, db):
    return

def send_chat_message(username, data, db):

    data_array = json.loads(data)

    if data == "" or data is None or data_array is None or "msg" not in data_array:
        return Status(type=STATUS_ERROR).get_dict()

    #if "img" in data_array:
        #Bild speichern

        #Bild in JSON-Daten einfügen

    timestamp = time.time()*1000

    with db:
        cur = db.cursor()
        cur.execute("INSERT INTO CHAT_DATA (USERNAME, DATA, TIMESTAMP) VALUES (:name, :data, :timestamp)",
            {'name': username, 'data': json.dumps(data_array), 'timestamp': timestamp})

        cur.execute("SELECT USERNAME FROM USERDATA WHERE USERNAME != :user", {'user': username})
        users = []

        for user in cur.fetchall():
            users.append(user['USERNAME'])

        send_notification_to_users(users, "Neue Nachricht von "+username, json.loads(data)['msg'], db, click_action="ChatActivity")

        cur.close()

    return {'username': username, 'data': data_array, 'time': timestamp}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.Modules.ShoppingListModule import VoiceShoppingListModule
from homevee.VoiceAssistant.voice_patterns import *
from homevee.Functions import shopping_list
from homevee.VoiceAssistant.helper import generate_string

class VoiceGetShoppingListModule(VoiceShoppingListModule):
    def get_pattern(self, db):
        return PATTERN_GET_SHOPPING_LIST

    def get_label(self):
        return "addshoppinglist"

    def run_command(self, username, text, context, db):
        return self.get_shopping_list(username, text, context, db)

    def get_shopping_list(self, username, text, context, db):
        items = self.find_items(text, db)

        if len(items) is 0:
            # Ganze Liste abfragen
            items = shopping_list.get_shopping_list(username, db)['items']

        data = [
            [['Das ', 'Folgendes '], 'steht ', 'auf ',
             [[['der ', 'deiner '], 'Einkaufsliste'], [['dem ', 'deinem '], 'Einkaufszettel']], ': '],
            [['Diese ', 'Folgende '], ['Artikel ', 'Produkte '], ['stehen ', 'sind '], 'auf ',
             [[['der ', 'deiner '], 'Einkaufsliste'],
              [['dem ', 'deinem '], 'Einkaufszettel']], ': ']
        ]

        output = generate_string(data)

        for i in range(0, len(items)):
            item = items[i]

            if len(items) > 1:
                # Mehr als ein Element
                if i is len(items) - 1:
                    # Letztes Element
                    output = output + " und "
                elif i < len(items) - 1 and i > 0:
                    # Nicht erstes und nicht letztes Element
                    output = output + ", "

            amount_string = str(item['amount'])
            if(item['amount'] == 1):
                amount_string = "ein"

            output = output + amount_string + " mal " + item['item']

        output += "."

        return {'msg_speech': output, 'msg_text': output}




#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.get_devices import set_device_value
from homevee.DeviceAPI.zwave.utils import do_zwave_request
from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_database_con
from homevee.utils.device_types import *

class ZWaveValueLoaderCronjob(IntervalCronjob):
    def __init__(self):
        super(ZWaveValueLoaderCronjob, self).__init__(task_name="ZWaveValueLoaderCronjob", interval_seconds=5*60)

    def task_to_do(self, *args):
        self.load_zwave_values()

    def load_zwave_values(self):
        db = get_database_con()

        with db:
            cur = db.cursor()

            # Alle Z-Wave Gerätetypen durchlaufen und den aktuellen Wert in Datenbank schreiben

            #Stromzähler
            TYPE = ZWAVE_POWER_METER
            cur.execute("SELECT * FROM ZWAVE_POWER_METER")
            data = cur.fetchall()

            for item in data:
                ID = item['DEVICE_ID']
                result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

                if result is None or result['code'] != 200:
                    value = "N/A"
                else:
                    value = result['data']['metrics']['level']

                set_device_value(TYPE, ID, value, db)

            #Thermostat
            TYPE = ZWAVE_THERMOSTAT
            cur.execute("SELECT * FROM ZWAVE_THERMOSTATS")
            data = cur.fetchall()

            for item in data:
                ID = item['THERMOSTAT_ID']
                result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

                if result is None or result['code'] != 200:
                    value = "N/A"
                else:
                    value = result['data']['metrics']['level']

                set_device_value(TYPE, ID, value, db)

            #Sensor
            TYPE = ZWAVE_SENSOR
            cur.execute("SELECT * FROM ZWAVE_SENSOREN")
            data = cur.fetchall()

            for item in data:
                ID = item['ID']
                result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

                if result is None or result['code'] != 200:
                    value = "N/A"
                else:
                    value = result['data']['metrics']['level']

                    #if item['VALUE'] != value:
                        #trigger automation

                set_device_value(TYPE, ID, value, db)

            #Schalter
            TYPE = ZWAVE_SWITCH
            cur.execute("SELECT * FROM ZWAVE_SWITCHES")
            data = cur.fetchall()

            for item in data:
                ID = item['ID']
                result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

                if result is None or result['code'] != 200:
                    value = "N/A"
                else:
                    value = result['data']['metrics']['level']

                    value = value == "on"

                set_device_value(TYPE, ID, value, db)

        db.close()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.blind_control import get_blinds
from homevee.DeviceAPI.dimmer import get_dimmers
from homevee.DeviceAPI.energy_data import get_power_usage_devices
from homevee.DeviceAPI.get_modes import get_modes
from homevee.DeviceAPI.heating import get_thermostats
from homevee.DeviceAPI.rgb_control import get_rgb_devices
from homevee.DeviceAPI.toggle_devices import get_toggle_devices
from homevee.DeviceAPI.wake_on_lan import get_xbox_devices, get_wol_devices
from homevee.Functions.sensor_data import get_sensor_data
from homevee.Functions.triggers import get_triggers
from homevee.Helper import Logger
from homevee.Manager.scenes import get_scenes
from homevee.items.Room import Room
from homevee.items.Status import *
from homevee.utils.device_types import *


def get_rooms(user, db):
    rooms = []
    all_rooms = Room.load_all(db)
    for room in all_rooms:
        if(user.has_permission(room.id)):
            rooms.append(room)
    return {'rooms': Room.list_to_dict(rooms)}

def get_room_data(user, room, db):
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    roomdata = []

    #Schalter abfragen
    switchdata = get_modes(user, room, "", "", db)
    for item in switchdata['modi']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['device_type'],
            'icon': item['icon'], 'type': 'switch', 'value': item['mode']})

    #Dimmer
    dimmerdata = get_dimmers(user, room, db)
    for item in dimmerdata['dimmer']:
        roomdata.append({'name': item['name'], 'id': item['device'], 'devicetype': item['type'],
                         'icon': item['icon'], 'type': 'dimmer', 'value': item['value']})
    #Toggle
    toggledata = get_toggle_devices(user, room, db)
    for item in toggledata['toggles']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
            'icon': item['icon'], 'type': 'toggle', 'value': ''})

    #Trigger
    triggerdata = get_triggers(user, room, db)
    for item in triggerdata['triggers']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
            'icon': item['icon'], 'type': 'trigger', 'value': ''})

    #RGB
    rgbdata = get_rgb_devices(user, room, db)
    for item in rgbdata['rgb']:
        Logger.log(item)
        roomdata.append({'name': item['name'], 'devicetype':item['type'], 'id': item['id'],
            'icon': item['icon'], 'type': 'rgb', 'value': item['value']})

    #Thermostate
    thermostat_data = get_thermostats(user, room, "", "", db)
    for item in thermostat_data['thermostats'][0]['thermostat_array']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                         'icon': item['icon'], 'type': 'heating', 'value': item['data']['value'],
                         'step': 0.5, 'min': item['data']['min'], 'max': item['data']['max']})

    #Sensoren abfragen
    sensordata = get_sensor_data(user, room, "", "", True, db)

    #print sensordata

    for item in sensordata['values'][0]['value_array']:
        roomdata.append({'name': item['shortform'], 'id': item['id'], 'devicetype': item['device_type'],
                         'icon': item['icon'], 'type': "value", 'value': item['wert']})

    #Szenen abfragen
    scenedata = get_scenes(user, room, db)
    if len(scenedata['scenes']) > 0:
        roomdata.append({'name': "Szenen", 'id': "scenes", 'devicetype': "scenes",
                         'icon': "scenes", 'type': "scenes", 'value': ""})

    #WOL abfragen
    wol_devices = get_wol_devices(user, room, db)

    for item in wol_devices['devices']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': WAKE_ON_LAN,
                         'icon': item['icon'], 'type': 'wakeonlan'})

    #XBOX ONE WOL abfragen
    xbox_devices = get_xbox_devices(user, room, db)

    for item in xbox_devices['devices']:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': XBOX_ONE_WOL,
                         'icon': item['icon'], 'type': 'xboxone'})

    #Jalousie-Steuerung
    blind_controls = get_blinds(user, room, db)['blinds']
    for item in blind_controls:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                         'icon': item['icon'], 'type': 'blinds', 'value': item['value']})

    #Strom-Messgeräte
    power_usage_devices = get_power_usage_devices(user, room, db)
    for item in power_usage_devices:
        roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['devicetype'],
                         'icon': item['icon'], 'type': 'value', 'value': item['value']})

    return {'roomdata': roomdata}

'''<?php
	
    //Neu dazugekommen
	$reeddata = getReedData($room, "", "", $db);
    foreach($reeddata['values'][0]['value_array'] as $reed){
		//Die Werte des Arrays wurden angepasst
        array_push($roomdata, array('name' => $reed['shortform'], 'device' => $reed['id'],
		'icon' => $reed['icon'], 'type' => "reed", 'device_type' => $reed['device_type'], 'value' => $reed['wert']));
    }
	
	$presencedata = getPresenceData($room, "", "", $db);
    foreach($presencedata['values'][0]['value_array'] as $presence){
        array_push($roomdata, array('name' => $presence['shortform'], 'device' => $presence['id'],
		'icon' => $presence['icon'], 'type' => "presence", 'device_type' => $presence['device_type'], 'value' => $presence['wert']));
    }
	
    $diydata = getDIYDevices($room, $db);
    foreach($diydata['diydevices'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['device'], 'icon' => $device['icon'],
		'type' => 'diydevice', 'value' => "", 'device_type' => "DIY Gerät"));
    }
	
    $dimmer = getDimmers($room, $db);
    foreach($dimmer['dimmers'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['id'], 'icon' => $device['icon'],
		'type' => 'dimmer', 'value' => $device['value'], 'device_type' => $device['device_type']));
    }
	
    $powermeters = getPowerMeterData($room, "", "", "", $db);
	
    foreach($powermeters['powermeters'][0]['value_array'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['id'], 'icon' => $device['icon'],
		'type' => 'powermeter', 'value' => $device['value'], 'device_type' => $device['device_type']));
    }
	
	//IP-Kameras abfragen
	$ipcameras = getIpCameraData($room, "", $db);
	
    foreach($ipcameras['ipcameras'][0]['value_array'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['id'], 'icon' => $device['icon'],
		'type' => 'ipcamera', 'value' => $device['value'], 'device_type' => $device['device_type'],
		'url' => $device['url'], 'username' => $device['username'], 'password' => $device['password']));
    }
	
    //Thermostate
	$thermostats = getHeatingData($room, "", "", "1", $db);
	
    foreach($thermostats['thermostats'][0]['thermostat_array'] as $thermostat){
        array_push($roomdata, array('name' => $thermostat['name'], 'device' => $thermostat['id'],
		'icon' => $thermostat['icon'], 'type' => "heating", 'device_type' => $thermostat['device_type'], 'value' => $thermostat['data']['value'],
		'einheit' => $thermostat['data']['einheit'], 'min' => $thermostat['data']['min'], 'max' => $thermostat['data']['max'], 'step' => 0.5));
    }
	
	//WOL-Geräte
	$devices = getWakeOnLanDevices($room, $db);
	
    foreach($devices['devices'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['id'],
		'icon' => $device['icon'], 'type' => "wakeonlan", 'device_type' => "Wake on Lan Gerät", 'value' => "Wake on Lan"));
    }
	
	//XBOX One
	$devices = getXboxOneDevices($room, $db);
	
    foreach($devices['devices'] as $device){
        array_push($roomdata, array('name' => $device['name'], 'device' => $device['id'],
		'icon' => $device['icon'], 'type' => "xboxone", 'device_type' => "XBOX One", 'value' => "Hochfahren"));
    }
     
    return array('roomdata' => $roomdata);
}
 
?>'''


from platform import system as system_name  # Returns the system/OS name
from subprocess import call   as system_call  # Execute a shell command

from homevee.Helper import Logger
from homevee.cronjobs import IntervalCronjob
from homevee.utils import database


class UserPingCronjob(IntervalCronjob):
    def __init__(self):
        super(UserPingCronjob, self).__init__(task_name="UserPingCronjob", interval_seconds=15)

    def task_to_do(self, *args):
        #self.ping_users()
        pass

    def ping_users(self):
        db = database.get_database_con()

        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM USERDATA")

            items = cur.fetchall()

            for item in items:
                ip = item['IP']

                Logger.log("Pinging user: "+item['USERNAME'])

                last_val = item['AT_HOME']

                at_home = self.ping(ip)

                #if last_val != at_home:
                #   trigger automation

                if at_home:
                    Logger.log(item['USERNAME']+ ": at home")
                else:
                    Logger.log(item['USERNAME']+ ": not at home")

                cur.execute("UPDATE USERDATA SET AT_HOME = :at_home WHERE USERNAME = :user",
                            {'at_home': at_home, 'user': item['USERNAME']})

    def ping(self, host):
        """
        Returns True if host (str) responds to a ping request.
        Remember that a host may not respond to a ping (ICMP) request even if the host name is valid.
        """

        # Ping command count option as function of OS
        param = '-n' if system_name().lower()=='windows' else '-c'

        # Building the command. Ex: "ping -c 1 google.com"
        command = ['ping', param, '1', host]

        # Pinging
        return system_call(command) == 0


#!/usr/bin/python
# -*- coding: utf-8 -*-
import zlib


def compress_string(data):
    deflate_compress = zlib.compressobj(9, zlib.DEFLATED, -zlib.MAX_WBITS)
    #zlib_compress = zlib.compressobj(9, zlib.DEFLATED, zlib.MAX_WBITS)
    #gzip_compress = zlib.compressobj(9, zlib.DEFLATED, zlib.MAX_WBITS | 16)

    deflate_data = deflate_compress.compress(data) + deflate_compress.flush()
    #zlib_data = zlib_compress.compress(data) + zlib_compress.flush()
    #gzip_data = gzip_compress.compress(data) + gzip_compress.flush()

    return deflate_data

def decompress_string(data):
    print(data)

    return zlib.decompress(data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission
from homevee.items.Room import Room
from homevee.items.Status import *


def get_automations(username, room, db):
    if not has_permission(username, room, db):
        return {'result': 'nopermission'}

    rules = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM AUTOMATION_DATA WHERE LOCATION = :location", {'location': room})

        for item in cur.fetchall():
            data = get_full_automation_data(item['ID'], db)

            if 'result' in data and data['result'] == 'nopermission':
                continue

            rules.append(data)

    return {'rules': rules}

def get_full_automation_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM AUTOMATION_DATA WHERE ID = :id", {'id': id})

        data = cur.fetchone()

        trigger_data = get_trigger_data(id, db)

        return {'name': data['NAME'], 'id': data['ID'], 'location': data['LOCATION'],
                'locationname': Room.get_name_by_id(data['LOCATION'], db), 'triggerdata': trigger_data,
                'conditiondata': data['CONDITION_DATA'], 'actiondata': data['ACTION_DATA'], 'isactive': True}

def get_trigger_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

        items = cur.fetchall()

        trigger_data = []

        for item in items:
            trigger_data.append({'type': item['TYPE'], 'id': item['ID'], 'value': item['VALUE'], 'text': item['TEXT']})

        return trigger_data

def add_edit_automation_rule(username, location, id, name, trigger_data, condition_data, action_data, is_active, db):
    if not has_permission(username, location, db):
        return {'result': 'nopermission'}

    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if(add_new):
            cur.execute("INSERT INTO AUTOMATION_DATA (LOCATION, NAME, CONDITION_DATA, ACTION_DATA, IS_ACTIVE) VALUES (:location, :name, :conditions, :actions, :active)",
                        {'location': location, 'name': name, 'conditions': condition_data, 'actions': action_data, 'active': is_active})

            id = cur.lastrowid

            trigger_data = json.loads(trigger_data)

            add_trigger_data(trigger_data, id, db)
            return Status(type=STATUS_OK).get_dict()

        else:
            cur.execute("UPDATE AUTOMATION_DATA SET LOCATION = :location, NAME = :name, CONDITION_DATA = :conditions, ACTION_DATA = :actions, IS_ACTIVE = :active WHERE ID = :id",
                {'location': location, 'name': name, 'conditions': condition_data, 'actions': action_data, 'active': is_active, 'id': id})

            trigger_data = json.loads(trigger_data)

            cur.execute("DELETE FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

            add_trigger_data(trigger_data, id, db)
            return Status(type=STATUS_OK).get_dict()

def delete_automation_rule(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM AUTOMATION_DATA WHERE ID = :id", {'id': id})

        cur.execute("DELETE FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

        return Status(type=STATUS_OK).get_dict()

def add_trigger_data(data, id, db):
    with db:
        cur = db.cursor()
        for data in data:
            param_array = {'rule': id, 'type': data['type'], 'text': data['textdata']}

            if('id' in data):
                param_array['id'] = data['id']
            else:
                param_array['id'] = None

            if('value' in data):
                param_array['value'] = data['value']
            else:
                param_array['value'] = None

            cur.execute(
                "INSERT INTO AUTOMATION_TRIGGER_DATA (AUTOMATION_RULE_ID, TYPE, ID, VALUE, TEXT) VALUES (:rule, :type, :id, :value, :text)",
                param_array)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.utils.colors import COLORS
from homevee.utils.device_types import *

PATTERN_SET_MODES = [
    [['mach', 'schalt'], ['an','ein','aus']]
]

PATTERN_SET_THERMOSTAT = [
    [['mach', 'stell', 'dreh'], ['thermostat', 'temperatur', 'heizung', 'heizkörper'],['aus','auf']]
]

PATTERN_CONTROL_BLINDS = [
    [['mach', 'stell', 'fahr'], ['jalousie', 'jalousien', 'rollladen', 'rollläden', 'rolladen', 'rolläden', 'rollo'],
     ['rauf', 'runter', 'herunter', ['nach', 'oben'], ['nach', 'unten'], 'hoch', 'zu','auf']]
]

PATTERN_SET_DIMMER = [
    [['mach', 'stell', 'dreh'],['aus','auf'],'prozent']
]

PATTERN_CALCULATOR = [
    ['wie', 'viel',['ist','ergibt']],
    ['was',['ist','ergibt']]
]

PATTERN_JOKE = [
    [['erzähl','sag'],'witz'],
    ['klopf', 'klopf']
]

PATTERN_GET_WEATHER = [
    ['wie',['ist','wird'],'wetter'],
    ['regnet','es'],
    ['gibt','regen'],
    ['wird','regnen']
]

PATTERN_GET_TV_SCHEDULE = [
    ['was',['kommt','ist','gibt','läuft'],['tv','fernsehen','fernseher']]
]

PATTERN_TV_TIPPS = [
    [['was', 'welche'], 'sind', ['tv-tipps', 'tvtipps', 'tv-tips', 'tvtips']],
    [['was', 'welche'], 'sind', ['tv', 'fernseh'], ['tipps', 'tips']],
    [['was', 'welche'], 'sind', ['fernseh-tipps', 'fernsehtipps', 'fernseh-tips', 'fernsehtips']],
]

PATTERN_GET_CALENDAR = [
    ['was', 'steht', 'an'],
    ['was', 'habe', 'ich', 'vor'],
    ['habe', 'ich', 'termine']
]

PATTERN_ADD_CALENDAR = [
    ['erinnere', 'mich']
]

PATTERN_GET_SHOPPING_LIST = [
    ['was', ['steht', 'ist'], ['einkaufszettel', 'einkaufsliste']],
    ['was', ['muss', 'soll'], 'kaufen'],
    ['wie', ['viel', 'viele'], ['muss', 'soll'], 'kaufen']
]

PATTERN_ADD_SHOPPING_LIST = [
    [['schreibe', 'setze'], ['auf', 'zur', 'in'], ['einkaufsliste','einkaufliste','einkaufszettel']]
]

PATTERN_PLACES = [
    [['wo', 'was'], 'ist', ['der', 'die', 'das'], ['nähste', 'nächste']]
]

PATTERN_SUMMARY = [
    ['was', 'so', 'los'],
    ['was', 'gibt']
]

PATTERN_RECIPES = [
    ['zeig', 'mir', 'rezepte', ['über', 'für', 'zu', 'von']]
]

PATTERN_GET_WIKIPEDIA_DEFINITION = [
    [['wer', 'was'], ['ist', 'war']]
]

PATTERN_GET_MOVIE_RATING = [
    ['wie', 'ist', 'film'],
    ['wie', ['gut', 'schlecht'], 'ist', 'film'],
    ['ist', 'film', ['gut', 'schlecht']]
]

PATTERN_DATE_WEEKDAY = [
    ['was', 'ist', 'für', ['tag', 'wochentag']]
]

PATTERN_ROUTE = [
    ['wie', 'weit', 'ist', ['bis', 'zu', 'nach']],
    ['wie', ['lang', 'lange'], ['braucht', 'brauche'], ['bis', 'zu', 'nach']]
]

SENSOR_ADJECTIVES = ['warm', 'kalt', 'dunkel', 'hell', 'feucht', 'laut']
SENSOR_SUBJECTIVES = ['temperatur', 'helligkeit', 'luftfeuchtigkeit', 'lautstärke', 'verbrauch', 'stromverbrauch']
PATTERN_SENSOR_DATA = [
    ['wie', SENSOR_ADJECTIVES, 'ist'],
    ['wie', 'ist', ['der', 'die', 'das'], SENSOR_SUBJECTIVES]
]

PATTERN_REED_SENSOR_DATA = [
    [['ist', 'sind'], ['tür', 'türen', 'fenster'], ['geschlossen', 'zu', 'geöffnet', 'offen', 'auf']]
]

PATTERN_PRESENCE_SENSOR_DATA = [
    ['ist', 'jemand', ['in', 'im']]
]

PATTERN_ACTIVITY = [
    [['mir', 'uns'], 'ist', 'langweilig'],
    ['was', ['kann', 'können'], ['ich', 'wir'], ['tun', 'machen', 'unternehmen']]
]

PATTERN_WOL = [
    ['fahr', 'hoch']
]

PATTERN_RGB_CONTROL = [
    [['mach', 'stell', 'tu', 'dreh'], COLORS]
]

PATTERN_ADD_NUTRITION_ITEM = [
    ['ich', ['gegessen', 'essen', 'esse', 'getrunken', 'trinke', 'trink']]
]

PATTERN_GET_NUTRITION_INFO = [
    ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['hat', 'ist', 'haben', 'sind']]
]

PATTERN_GET_NUTRITION_DIARY = [
    ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['ist', 'sind', 'habe'], ['übrig', 'offen', 'auf', 'erlaubt']],
    ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['darf', 'kann', 'soll', 'muss'], ['essen', 'trinken', 'verzehren', ['zu', 'mir', 'nehmen']]]
]

def GET_PATTERN_SET_MODES(db):
    with db:
        cur = db.cursor()
        data = [
            ['FUNKSTECKDOSEN','NAME','DEVICE',FUNKSTECKDOSE],
            ['ZWAVE_SWITCHES','NAME','ID',ZWAVE_SWITCH],
            ['URL_SWITCH_BINARY','NAME','ID',URL_SWITCH],
            ['URL_TOGGLE','NAME','ID',URL_TOGGLE],
            ['PHILIPS_HUE_LIGHTS','NAME','ID',PHILIPS_HUE_LIGHT],
            ['URL_RGB_LIGHT','NAME','ID',URL_RGB_LIGHT]
        ]
        device_names = []
        for item in data:
            cur.execute("SELECT * FROM "+item[0])
            for device in cur.fetchall():
                device_names.append(device[item[1]].lower())
    output_array = [
        [['mach', 'schalt'], device_names, ['an','ein','aus']],
        [device_names, ['an','ein','aus']]
    ]
    #print output_array
    return output_array

def GET_PATTERN_WOL(db):
    with db:
        cur = db.cursor()
        data = [
            ['WAKE_ON_LAN','NAME','DEVICE',WAKE_ON_LAN],
            ['XBOX_ONE_WOL','NAME','ID',XBOX_ONE_WOL]
        ]
        device_names = []
        for item in data:
            cur.execute("SELECT * FROM "+item[0])
            for device in cur.fetchall():
                device_names.append(device[item[1]].lower())
    output_array = [
        [['fahr'], device_names, ['hoch']]
    ]
    return output_array



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import time
import traceback
from time import sleep

import paho.mqtt.client as mqtt

from homevee.DeviceAPI.mqtt_sensor import handle_mqtt_sensor
from homevee.Helper import Logger, smarthome_functions, translations
from homevee.items.Gateway import *
from homevee.utils.database import get_database_con

MQTT_CLIENT = None

topics_to_subscribe = [
    '/home/#'
]

QUALITY_OF_SERVICE = 0

# https://www.dinotools.de/2015/04/12/mqtt-mit-python-nutzen/

def run_device_action(topic, msg, db):
    topic_parts = topic.split("/")

    #topic_parts[0] is empty
    #topic_parts[1] is "home"

    result = None

    if(topic_parts[2] == "device"):
        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM MQTT_DEVICES WHERE TOPIC = :topic",
                        {'topic': topic})

            result = cur.fetchone()

            key = result['KEY']

            # aes = AESCipher.AESCipher(key)
            # msg = aes.decrypt(msg)

            with db:
                cur = db.cursor()

                data = json.loads(msg)

                device_types = ["MQTT_SENSORS"]  # , "MQTT_TRIGGERS"]

                for device_type in device_types:
                    cur.execute("SELECT * FROM " + device_type + " WHERE DEVICE_ID = :dev_id",
                                {'dev_id': result['ID']})

                    result = cur.fetchall()

                    for device in result:
                        for data_item in data:
                            if str(device['VALUE_ID']) == str(data_item['id']):
                                handle_mqtt_sensor(device['ID'], data_item['value'], db)

                # Run MQTT-Trigger
                '''cur.execute("SELECT * FROM MQTT_TRIGGERS WHERE TOPIC = :topic", {'topic': message.topic})
                for item in cur.fetchall():
                    #handle item action
                    data = json.loads(msg)
                    Logger.log(data)
                    Logger.log(item)
                    run_trigger_automation("MQTT-Trigger", item['TYPE'], item['ID'], data['action'], db)
                    break'''
    elif(topic_parts[2] == "assistant" and topic_parts[4]=="send"):
        start_time = time.time()

        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM SMART_SPEAKER WHERE ID = :id",
                        {'id': topic_parts[3]})

            smart_speaker = cur.fetchone()

            topic = '/' + topic_parts[1] + '/' + topic_parts[2] + '/' + topic_parts[3] + '/receive'

            try:
                data = json.loads(msg)

                data = json.loads(data['msg'])

                username = data['username']
                text = data['text']

                text = text.encode('utf-8')

                result = smarthome_functions.do_voice_command(username, text, None, smart_speaker['LOCATION'], db, translations.LANGUAGE)

                result['time'] = time.time()
            except:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()
                answer = "Es gab einen Fehler."
                result = {'msg_text': answer, 'msg_speech': answer}

            end_time = time.time()
            result['computing_duration'] = end_time - start_time

            publish(topic, json.dumps(result))

def publish(topic, msg):
    client = mqtt.Client()

    gateway = Gateway.load_from_db(MQTT_BROKER, get_database_con())

    Logger.log("publishing to "+topic+": "+msg)

    client.connect(gateway.ip)
    client.publish(topic, msg, QUALITY_OF_SERVICE, False)

def on_message(client, userdata, message):
    db = get_database_con()

    msg = str(message.payload.decode("utf-8"))
    Logger.log("message received: ", msg)
    Logger.log("message topic: ", message.topic)
    # Logger.log("message qos=",message.qos)
    # Logger.log("message retain flag=",message.retain)

    # decrypt message with saved key of the device in db
    try:
        run_device_action(message.topic, msg, db)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

def on_connect(client, userdata, flags, rc):
    for topic in topics_to_subscribe:
        client.subscribe(topic, QUALITY_OF_SERVICE)

    for topic in get_topics():
        client.subscribe(topic, QUALITY_OF_SERVICE)


def init_client():
    while (True):
        gateway = Gateway.load_from_db(MQTT_BROKER, get_database_con())

        if gateway is None:
            sleep(30)

        try:
            broker_address = gateway.ip
        except:
            broker_address = None

        #keep retrying when broker_address is None
        if broker_address is None:
            sleep(15)
            continue

        try:
            client = mqtt.Client()
            client.on_connect = on_connect
            client.on_message = on_message

            client.connect(broker_address)

            Logger.log("Connected to MQTT Broker: " + broker_address)


            while(True):
                #check if client still connected and exit if not
                client.loop(1)

        except socket.error as e:
            continue
        # Logger.log("Cannot reach MQTT Broker: " + broker_address)


def get_topics():
    # load topics from database

    return []



#!/usr/bin/python
# -*- coding: utf-8 -*-

class VoiceModule():
    def __init__(self, priority=1):
        self.priority = priority

    def get_context_key(self):
        return None

    def get_priority(self):
        return self.priority

    def get_pattern(self, db):
        return None

    def get_label(self):
        return None

    def run_command(self, username, text, context, db):
        pass

    #helper functions
    def is_number(self, s):
        try:
            float(s)
            return True
        except ValueError:
            return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib.error
import urllib.parse
import urllib.request

from homevee.items.Gateway import *


def set_modes(id, mode, db):
    gateway = Gateway.load_from_db(RADEMACHER_HOMEPILOT, db)

    url = "http://" + gateway.ip + "/deviceajax.do?cid=9&did=10001&goto=" + str(mode) + "&command=0"

    response = urllib.request.urlopen(url).read()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request
import xml.etree.ElementTree as etree

from homevee.items.Status import *


def get_tv_plan(username, type, db):
    tv_channels = get_tv_channels(username, db)

    if type == "2015":
        type = "heute2015"
    elif type == "2200":
        type = "heute2200"
    elif type == "jetzt":
        type = "jetzt"
    elif type == "tipps":
        type = "tipps"
    else:
        raise AttributeError("TV-Typ '" + type + "' nicht vorhanden")

    tv_shows = []

    link = "http://www.tvspielfilm.de/tv-programm/rss/" + type + ".xml"

    file = urllib.request.urlopen(link)
    data = file.read()
    file.close()

    #XML-Datei laden
    ergebnis = etree.fromstring(data)

    for item in ergebnis.find('channel').findall('item'):
        zeit, channel, name = item.find('title').text.split(" | ")

        description = item.find('description').text

        img = None
        if item.find('image') is not None:
            img = item.find('enclosure').attrib['url']

        if type == "tipps" or ((tv_channels is None) or (channel in tv_channels)):
            tv_shows.append({'time': zeit, 'channel': channel, 'name': name, 'description': description, 'img': img})

    return tv_shows

def get_tv_channels(username, db):
    cur = db.cursor()
    cur.execute("SELECT * FROM TV_CHANNELS WHERE USERNAME == :username", {'username': username})

    channels = []

    for channel in cur.fetchall():
        channels.append(channel['CHANNEL'])

    return channels

def get_all_tv_channels(username, db):
    selected_channels = get_tv_channels(username, db)

    #XML-Datei von Link laden
    link = "http://www.tvspielfilm.de/tv-programm/rss/jetzt.xml"

    file = urllib.request.urlopen(link)
    data = file.read()
    file.close()

    # XML-Datei laden
    ergebnis = etree.fromstring(data)

    channels = []
    for item in ergebnis.find('channel').findall('item'):
        zeit, channel, name = item.find('title').text.split(" | ")

        if (channels is None) or (channel not in channels):
            channels.append({'name': channel, 'selected': (channel in selected_channels)})

    return channels

def set_tv_channels(username, json_data, db):
    cur = db.cursor()
    cur.execute("DELETE FROM TV_CHANNELS WHERE USERNAME == :username", {'username': username})

    #Abfrage erfolgreich?
    if False:
        return Status(type=STATUS_ERROR).get_dict()
    else:
        channels = json.loads(json_data)

        for channel in channels:
            cur.execute("INSERT INTO TV_CHANNELS (USERNAME, CHANNEL) VALUES (?,?)",
                [username, channel])

        return Status(type=STATUS_OK).get_dict().get_dict()


from homevee.VoiceAssistant.Modules import VoiceModule

class VoiceShoppingListModule(VoiceModule):
    def find_items(self, text, db):
        items = []

        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM SHOPPING_LIST")

            for item in cur.fetchall():
                position = text.find(item['ITEM'].lower())
                if position is not -1:
                    items.append({'item': item['ITEM'], 'amount': item['AMOUNT'], 'id': item['ID']})

        return items


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from homevee.Helper.helper_functions import has_permission
from homevee.items.Gateway import *
from homevee.items.Status import *


def add_edit_gateway(username, type, user, password, change_pw, ip, port, gateway_type, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    param_array = {'name': type, 'ip': ip, 'port': port, 'key1': user, 'type': gateway_type}

    with db:
        cur = db.cursor()

        if change_pw == "true":
            param_array['key2'] = password

            cur.execute("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1, KEY2 = :key2 TYPE = :type, WHERE NAME = :name;",
                        param_array)

            cur.execute("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, KEY2, TYPE) VALUES (:name, :ip, :port, :key1, :key2, :type);",
                        param_array)
        else:
            cur.execute("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1 WHERE NAME = :name;",
                        param_array)

            cur.execute("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, TYPE) VALUES (:name, :ip, :port, :key1, :type);",
                        param_array)

    return Status(type=STATUS_OK).get_dict()

def delete_gateway(user, key, db):
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    gateway = Gateway.load_from_db(key)

    try:
        if(gateway.delete()):
            return Status(type=STATUS_OK).get_dict()
    except:
            return Status(type=STATUS_ERROR).get_dict()

    '''
    with db:
        cur = db.cursor()
        cur.execute("DELETE FROM GATEWAYS WHERE NAME == :key", {'key': key})

        cur.close()

        #Abfrage erfolgreich?
        if True:
            return Status(type=STATUS_OK).get_dict()
        else:
            return Status(type=STATUS_ERROR).get_dict()
    '''

def get_gateways(username, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM GATEWAYS")

        gateways = []

        for gateway in cur.fetchall():
            gateways.append({'name': gateway['NAME'], 'ip': gateway['IP'], 'port': gateway['PORT'],
                             'key1': gateway['KEY1'], 'type': gateway['TYPE']})

        cur.close()

        return {'gateways': gateways, 'gatewaytypesleft': get_gateway_types(username, db)}

def get_gateway_types(username, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    gateway_types = [
        {'key': Z_WAVE_GATEWAY, 'type': 'user'},
        {'key': FUNKSTECKDOSEN_CONTROLLER, 'type': 'url'},
        {'key': MAX_CUBE, 'type': 'url'},
        {'key': MQTT_BROKER, 'type': 'user'},
        {'key': PHILIPS_HUE_BRIDGE, 'type': 'apikey'},
        {'key': MIYO_CUBE, 'type': 'apikey'},
        {'key': RADEMACHER_HOMEPILOT, 'type': 'url'},
    ]

    with db:
        cur = db.cursor()
        cur.execute("SELECT NAME FROM GATEWAYS")

        for gateway in cur.fetchall():
            name = gateway['NAME']
            for i in range(0, len(gateway_types)-1):
                if gateway_types[i]['key'] == name:
                    del gateway_types[i]

        cur.close()

        return gateway_types

def connect_gateway(username, type, ip, db):
    if not has_permission(username, "admin", db):
        return {'result': 'noadmin'}

    if type == PHILIPS_HUE_BRIDGE:
        while True:
            print("try connecting...")

            data = {"devicetype": "homevee#system"}

            response = requests.post("http://" + ip + "/api", data=json.dumps(data))

            result = response.text

            print("result: "+result)

            result = json.loads(result)

            result = result[0]

            if "error" in result:
                if result['error']['type'] == 101:
                    return {'result': 'error', 'msg': 'Drücke bitte die Taste auf deinem Philips Hue Gateway.'}
            elif "success" in result:
                user = result['success']['username']

                add_edit_gateway(username, type, user, "", "", ip, "80", "apikey", db)

                return Status(type=STATUS_OK).get_dict()
    elif type == MIYO_CUBE:
        while True:
            print("try connecting...")

            response = requests.get("http://" + ip + "/api/link")

            result = response.text

            print("result: "+result)

            result = json.loads(result)

            if "error" in result:
                if result['errorLoc'] == "NOTIFY_ERROR_LINKINACTIVE":
                    return {'result': 'error', 'msg': 'Drücke bitte die Taste hinten auf deinem MIYO Cube.'}
            elif "success" in result:
                user = result['success']['username']

                add_edit_gateway(username, type, user, "", "", ip, "80", "apikey", db)

                return Status(type=STATUS_OK).get_dict()
    else:
        return Status(type=STATUS_ERROR).get_dict()


from homevee.exceptions import InvalidParametersException, AbstractFunctionCallException


class Item():
    def __init__(self):
        pass

    def save_to_db(self, db=None):
        raise AbstractFunctionCallException("Item.save_to_db() is abstract")

    def build_dict(self):
        raise AbstractFunctionCallException("Item.build_dict() is abstract")

    def get_dict(self, fields=None):
        dict = self.build_dict()

        if (fields is None):
            return dict
        else:
            try:
                output_dict = {}

                for field in fields:
                    output_dict[field] = dict[field]

                return output_dict
            except:
                raise InvalidParametersException("InvalidParams given for get_dict()")

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        raise AbstractFunctionCallException("Item.load_all_ids_from_db() is abstract")

    @staticmethod
    def load_all_from_db(query, params, db=None):
        raise AbstractFunctionCallException("Item.load_all_from_db() is abstract")

    @staticmethod
    def load_from_db(id, db=None):
        raise AbstractFunctionCallException("Item.load_from_db() is abstract")

    @staticmethod
    def create_from_dict(dict):
        raise AbstractFunctionCallException("Item.create_from_dict() is abstract")

    @staticmethod
    def list_to_dict(list):
        data = []

        for item in list:
            data.append(item.get_dict())

        return data


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_PLACES

class VoicePlacesApiModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_PLACES

    def get_label(self):
        return "placesapi"

    def run_command(self, username, text, context, db):
        return self.get_places(username, text, context, db)

    def get_places(self, username, text, context, db):
        return {'msg_speech': 'Places', 'msg_text': 'Places'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sqlite3

# use encrypted databases
# http://charlesleifer.com/blog/encrypted-sqlite-databases-with-python-and-sqlcipher/
from homevee.utils import constants


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

'''Gibt eine Instanz der Datenbankverbindung zurück'''
def get_database_con():
    db_path = os.path.join(constants.DATA_DIR, "data.db")

    #init db if not yet existing
    if not os.path.isfile(db_path):
        open(db_path, "w+")
        init_db()

    #print db_path

    con = sqlite3.connect(db_path)
    con.text_factory = str
    con.row_factory = dict_factory

    return con

def get_server_data(key, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                    {'key': key})

        try:
            return cur.fetchone()['VALUE']
        except Exception as e:
            return None

def set_server_data(key, value, db):
    with db:
        cur = db.cursor()

        cur.execute("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES(:value, :key)",
                    {'value': value, 'key': key})

        cur.execute("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key",
                    {'value': value, 'key': key})

def init_db():
    command = """
    CREATE TABLE "API_KEYS" ( "SERVICE_NAME" TEXT NOT NULL, "API_KEY" TEXT DEFAULT NULL, PRIMARY KEY("SERVICE_NAME") );
    CREATE TABLE "AR_CONTROL_CLASSES" ( "ID" INTEGER NOT NULL, "NAME" TEXT NOT NULL, "LOCATION" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "CONTROL_DATA" TEXT );
    CREATE TABLE "AR_CONTROL_LEARNING_DATA" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "PATH" TEXT NOT NULL, "CONTROL_KEY" INTEGER DEFAULT NULL );
    CREATE TABLE "AUTOMATION_DATA" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "ACTIVE" BOOLEAN NOT NULL DEFAULT "true", "CONDITION_DATA" TEXT NOT NULL, "ACTION_DATA" TEXT NOT NULL, "LOCATION" INTEGER NOT NULL, "TRIGGERED" INTEGER NOT NULL DEFAULT 0, "IS_ACTIVE" INTEGER NOT NULL DEFAULT 0 );
    CREATE TABLE "AUTOMATION_TRIGGER_DATA" ( "AUTOMATION_RULE_ID" INTEGER NOT NULL, "TYPE" TEXT NOT NULL, "ID" TEXT, "VALUE" TEXT );
    CREATE TABLE "CALENDAR" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "START" TEXT NOT NULL, "END" TEXT, "NOTE" TEXT, "REPEAT" TEXT );
    CREATE TABLE "CALENDAR_PARTICIPANTS" ( "USERNAME" TEXT NOT NULL UNIQUE, "CALENDAR_ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE );
    CREATE TABLE "CHAT_DATA" ( "USERNAME" TEXT NOT NULL, "DATA" TEXT NOT NULL, "TIMESTAMP" DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP UNIQUE );
    CREATE TABLE "CUSTOM_VOICE_COMMANDS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "ACTION_DATA" TEXT NOT NULL );
    CREATE TABLE "CUSTOM_VOICE_COMMAND_RESPONSES" ( "COMMAND_ID" INTEGER NOT NULL, "RESPONSE" TEXT NOT NULL );
    CREATE TABLE "CUSTOM_VOICE_COMMAND_SENTENCES" ( "COMMAND_ID" INTEGER NOT NULL, "COMMAND" TEXT NOT NULL );
    CREATE TABLE "DEVICE_AVAILABILITY" ( "DEVICE_TYPE" TEXT NOT NULL, "DEVICE_ID" TEXT NOT NULL, "IS_ONLINE" BOOLEAN NOT NULL DEFAULT "false", PRIMARY KEY("DEVICE_TYPE","DEVICE_ID") );
    CREATE TABLE "DEVICE_ENERGY_DATA" ( "DEVICE_TYPE" TEXT NOT NULL, "DEVICE_ID" TEXT NOT NULL, "POWER_USAGE" NUMERIC NOT NULL, "DATE" DATETIME NOT NULL DEFAULT CURRENT_DATE, "LOCATION" TEXT NOT NULL );
    CREATE TABLE "DIY_DEVICES" ( "DEVICE" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT, "ROOM" INTEGER, "IP" TEXT, "ICON" TEXT, "HTTP_METHOD" TEXT NOT NULL DEFAULT "HTTP GET" );
    CREATE TABLE "DIY_MOTIONSENSORS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "RAUM" INTEGER NOT NULL, "ICON" TEXT NOT NULL DEFAULT "motion", "NAME" TEXT NOT NULL, "IP" TEXT NOT NULL, "CHANNEL" INTEGER, "HTTP_METHOD" TEXT NOT NULL DEFAULT "get" );
    CREATE TABLE "DIY_POWER_METER" ( "ROOM_ID" INTEGER NOT NULL, "DEVICE_ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "DEVICE_NAME" TEXT, "IP" TEXT, "HTTP_METHOD" TEXT NOT NULL DEFAULT "HTTP GET" );
    CREATE TABLE "DIY_PRESENCESENSORS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "RAUM" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "presence", "NAME" TEXT NOT NULL, "IP" TEXT NOT NULL, "CHANNEL" INTEGER, "HTTP_METHOD" TEXT NOT NULL DEFAULT "get" );
    CREATE TABLE "DIY_REEDSENSORS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "RAUM" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "reed", "NAME" TEXT NOT NULL, "IP" TEXT NOT NULL, "CHANNEL" INTEGER, "HTTP_METHOD" TEXT NOT NULL DEFAULT "get" );
    CREATE TABLE "DIY_SENSORS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "RAUM" INTEGER NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device", "IP" TEXT NOT NULL, "NAME" TEXT NOT NULL, "EINHEIT" INTEGER, "SAVE_DATA" BOOLEAN NOT NULL DEFAULT "true", "CHANNEL" INTEGER, "HTTP_METHOD" TEXT NOT NULL DEFAULT "get" );
    CREATE TABLE "DIY_SWITCHES" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device", "RAUM" INTEGER NOT NULL, "IP" TEXT NOT NULL, "CHANNEL" INTEGER, "HTTP_METHOD" TEXT NOT NULL DEFAULT "get" );
    CREATE TABLE "ENERGY_DATA" ( "POWER_USAGE" NUMERIC, "DATE" DATETIME NOT NULL DEFAULT CURRENT_DATE, PRIMARY KEY("DATE") );
    CREATE TABLE "EVENTS" ( "TIMESTAMP" INTEGER UNIQUE, "TEXT" TEXT, "TYPE" TEXT );
    CREATE TABLE "FUNKSTECKDOSEN" ( "ROOM" INTEGER NOT NULL, "DEVICE" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "HAUSCODE" TEXT NOT NULL, "STECKDOSENNUMMER" TEXT NOT NULL, "ZUSTAND" BOOLEAN NOT NULL DEFAULT 0, "ICON" TEXT NOT NULL DEFAULT "socket", "NAME" TEXT NOT NULL, CONSTRAINT "unq" UNIQUE("HAUSCODE","STECKDOSENNUMMER") );
    CREATE TABLE "GATEWAYS" ( "NAME" TEXT NOT NULL, "IP" TEXT DEFAULT "localhost", "PORT" INTEGER NOT NULL DEFAULT 80, "KEY1" TEXT, "KEY2" TEXT, "TYPE" TEXT NOT NULL DEFAULT "user", PRIMARY KEY("NAME") );
    CREATE TABLE "GPS_DATA" ( "USERNAME" TEXT NOT NULL, "TIMESTAMP" INTEGER NOT NULL, "LATITUDE" INTEGER NOT NULL, "LONGITUDE" INTEGER NOT NULL );
    CREATE TABLE "HEATING_SCHEME" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "TIME" DATETIME NOT NULL, "VALUE" NUMERIC NOT NULL, "ACTIVE" BOOLEAN NOT NULL );
    CREATE TABLE "HEATING_SCHEME_DAYS" ( "HEATING_SCHEME_ID" INTEGER NOT NULL, "WEEKDAY_ID" INTEGER NOT NULL );
    CREATE TABLE "HEATING_SCHEME_DEVICES" ( "ID" INTEGER NOT NULL, "TYPE" TEXT NOT NULL, "DEVICE_ID" TEXT NOT NULL, "LOCATION" TEXT );
    CREATE TABLE "HOMEPILOT_BLIND_CONTROL" ( "ID" INTEGER NOT NULL UNIQUE, "NAME" TEXT, "LOCATION" TEXT, "ICON" TEXT DEFAULT "blinds", "LAST_POS" TEXT, PRIMARY KEY("ID") );
    CREATE TABLE "HOMEPILOT_THERMOSTATS" ( "ID" INTEGER NOT NULL UNIQUE, "NAME" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "heating", "LAST_TEMP" TEXT NOT NULL DEFAULT "N/A", "ROOM" TEXT NOT NULL, PRIMARY KEY("ID") );
    CREATE TABLE "HOME_BUDGET_DATA" ( "DATE" TEXT NOT NULL, "AMOUNT" NUMERIC NOT NULL, "INFO" TEXT NOT NULL, "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE );
    CREATE TABLE "HTTP_DEVICE" ( "DEVICE" INTEGER NOT NULL, "NAME" TEXT NOT NULL, "LOCATION" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device", "IP" TEXT NOT NULL, "HTTP_METHOD" TEXT DEFAULT "HTTP GET", "USERNAME" TEXT, "PASSWORD" TEXT, "DEVICE_DATA" TEXT NOT NULL, PRIMARY KEY("DEVICE") );
    CREATE TABLE "IP_CAMERA" ( "TYPE" TEXT NOT NULL, "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "IP" TEXT NOT NULL, "PORT" TEXT, "PATH" TEXT, "USERNAME" TEXT, "PASSWORD" TEXT, "NAME" TEXT NOT NULL, "LOCATION" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "camera", "RECORD_FOOTAGE" BOOLEAN NOT NULL DEFAULT "true", "AUTO_DELETE" BOOLEAN NOT NULL DEFAULT "false", "MOTION_DETECTION_TRESHOLD" NUMERIC, "CROP_WIDTH" INTEGER DEFAULT NULL, "CROP_HEIGHT" INTEGER DEFAULT NULL, "CROP_X_OFFSET" INTEGER DEFAULT NULL, "CROP_Y_OFFSET" INTEGER DEFAULT NULL );
    CREATE TABLE "IP_CAMERA_RECORDINGS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "DATETIME" DATETIME NOT NULL, "VIDEOPATH" TEXT NOT NULL, "THUMBNAILPATH" TEXT NOT NULL, "FOOTAGE_ID" INTEGER NOT NULL, "DURATION" INTEGER, "MOTION_DETECTED" TEXT DEFAULT NULL );
    CREATE TABLE "MAX_THERMOSTATS" ( "RAUM" TEXT NOT NULL, "NAME" TEXT NOT NULL, "ID" TEXT NOT NULL UNIQUE, "ICON" TEXT NOT NULL DEFAULT "heating", "LAST_TEMP" TEXT NOT NULL DEFAULT "N/A", PRIMARY KEY("ID") );
    CREATE TABLE "MEDIA_CENTER" ( "TYPE" TEXT NOT NULL, "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "IP" TEXT NOT NULL, "NAME" TEXT NOT NULL, "LOCATION" INTEGER NOT NULL, "USERNAME" TEXT, "PASSWORD" TEXT, "PORT" TEXT );
    CREATE TABLE "MQTT_DEVICES" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "LOCATION" TEXT NOT NULL, "KEY" TEXT NOT NULL, "TOPIC" TEXT NOT NULL );
    CREATE TABLE "MQTT_DEVICE_INTERMEDIATES" ( "ID" INTEGER NOT NULL, "TYPE" TEXT NOT NULL );
    CREATE TABLE "MQTT_SENSORS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "LAST_VALUE" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device", "TYPE" TEXT NOT NULL, "ROOM" INTEGER NOT NULL, "SAVE_DATA" INTEGER NOT NULL DEFAULT 0, "DEVICE_ID" INTEGER NOT NULL, "VALUE_ID" INTEGER NOT NULL );
    CREATE TABLE "MQTT_TRIGGERS" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "TOPIC" TEXT NOT NULL, "TYPE" TEXT NOT NULL, "ICON" INTEGER NOT NULL DEFAULT "device", "LOCATION" INTEGER NOT NULL );
    CREATE TABLE "NUTRITION_DATA" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "USER" TEXT NOT NULL, "DATE" TEXT NOT NULL, "DAYTIME" TEXT NOT NULL DEFAULT "snacks", "FOOD_NAME" TEXT NOT NULL, "PORTIONSIZE" NUMERIC NOT NULL, "EATEN_PORTION_SIZE" NUMERIC NOT NULL, "PORTIONUNIT" TEXT NOT NULL, "CALORIES" INTEGER NOT NULL, "PROTEIN" NUMERIC NOT NULL, "CARBS" NUMERIC NOT NULL, "SUGAR" NUMERIC, "FAT" NUMERIC NOT NULL, "SATURATED" NUMERIC, "UNSATURATED" NUMERIC );
    CREATE TABLE "PEOPLE_DATA" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "NICKNAME" TEXT, "PHONE_NUMBER" TEXT, "ADDRESS" TEXT, "BIRTHDATE" TEXT, "LATITUDE" NUMERIC, "LONGITUDE" NUMERIC );
    CREATE TABLE "PEOPLE_LEARNING_DATA" ( "ID" INTEGER NOT NULL, "PATH" TEXT NOT NULL, "PERSON_ID" INTEGER, PRIMARY KEY("ID") );
    CREATE TABLE "PHILIPS_HUE_LIGHTS" ( "NAME" TEXT NOT NULL, "ID" INTEGER NOT NULL UNIQUE, "LOCATION" INTEGER NOT NULL, "IS_ON" INTEGER, "BRIGHTNESS" INTEGER, "SATURATION" INTEGER, "HUE" INTEGER, "ICON" TEXT DEFAULT "light", "TYPE" TEXT NOT NULL DEFAULT "color", PRIMARY KEY("ID") );
    CREATE TABLE "PLACES" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "ADDRESS" TEXT NOT NULL, "LATITUDE" NUMERIC NOT NULL, "LONGITUDE" NUMERIC NOT NULL );
    CREATE TABLE "PUSH_NOTIFICATION_TOKENS" ( "USERNAME" TEXT NOT NULL, "TOKEN" TEXT NOT NULL UNIQUE );
    CREATE TABLE "REQUEST_DATA" ( "REQUEST_ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "REQUEST_TIME" DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP, "RESPONSE" TEXT NOT NULL );
    CREATE TABLE "REQUEST_DATA_PARAMS" ( "REQUEST_ID" INTEGER NOT NULL, "PARAM_KEY" TEXT NOT NULL, "PARAM_VALUE" TEXT );
    CREATE TABLE "RFID_TAGS" ( "NAME" TEXT NOT NULL, "UUID" TEXT NOT NULL UNIQUE, "ACTION_TYPE" TEXT NOT NULL, "ACTION_ID" TEXT NOT NULL, PRIMARY KEY("UUID") );
    CREATE TABLE "ROOMS" ( "NAME" TEXT NOT NULL, "LOCATION" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "ICON" TEXT NOT NULL DEFAULT room );
    CREATE TABLE "ROOM_ENERGY_DATA" ( "ROOM_ID" INTEGER NOT NULL, "POWER_USAGE" NUMERIC, "DATE" DATETIME NOT NULL DEFAULT CURRENT_DATE );
    CREATE TABLE "SCENES" ( "NAME" TEXT NOT NULL, "ROOM" TEXT NOT NULL, "ACTION_DATA" TEXT NOT NULL, "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE );
    CREATE TABLE "SENSOR_DATA" ( "VALUE" NUMERIC, "DEVICE_ID" TEXT, "DATETIME" DATETIME, "DEVICE_TYPE" TEXT NOT NULL DEFAULT "error" );
    CREATE TABLE "SERVER_DATA" ( "KEY" TEXT NOT NULL, "VALUE" TEXT NOT NULL, PRIMARY KEY("KEY") );
    CREATE TABLE "SESSION_KEYS" ( "USERNAME" TEXT NOT NULL, "SESSION_KEY" TEXT NOT NULL, "KEY_CREATED" DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY("USERNAME") );
    CREATE TABLE "SHOPPING_LIST" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "AMOUNT" NUMERIC, "ITEM" TEXT NOT NULL );
    CREATE TABLE "SMART_SPEAKER" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "KEY" TEXT NOT NULL, "LOCATION" INTEGER NOT NULL, "NAME" TEXT NOT NULL DEFAULT "Smart Speaker" );
    CREATE TABLE "TV_CHANNELS" ( "USERNAME" TEXT NOT NULL, "CHANNEL" TEXT NOT NULL );
    CREATE TABLE "URL_RGB_LIGHT" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "URL" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "light", "LAST_COLOR" TEXT, "LOCATION" INTEGER NOT NULL );
    CREATE TABLE "URL_SWITCH_BINARY" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "URL_ON" TEXT NOT NULL, "URL_OFF" TEXT NOT NULL, "URL_GET_STATE" TEXT NOT NULL, "ICON" TEXT DEFAULT "device", "LOCATION" INTEGER NOT NULL );
    CREATE TABLE "URL_TOGGLE" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "TOGGLE_URL" TEXT NOT NULL, "ICON" TEXT DEFAULT "device", "LOCATION" INTEGER NOT NULL );
    CREATE TABLE "USERDATA" ( "USERNAME" TEXT NOT NULL UNIQUE, "PASSWORD" TEXT NOT NULL, "IP" TEXT, "AT_HOME" INTEGER, "EVENTS_LAST_CHECKED" INTEGER, "PERMISSIONS" TEXT, "DASHBOARD_DATA" TEXT, "PW_SALT" TEXT NOT NULL, "FCM_TOKEN" TEXT UNIQUE, "CURRENT_LOCATION" INTEGER NOT NULL DEFAULT -1, PRIMARY KEY("USERNAME") );
    CREATE TABLE "USER_FITNESS_DATA" ( "USERNAME" TEXT NOT NULL, "CALORIE_GOAL" INTEGER NOT NULL, "FAT_GOAL" NUMERIC, "SATURATED_GOAL" NUMERIC, "PROTEIN_GOAL" NUMERIC, "CARBS_GOAL" NUMERIC, "SUGAR_GOAL" NUMERIC, "WATER_GOAL" NUMERIC DEFAULT 2000, "BIRTHDATE" TEXT, "HEIGHT" INTEGER, "DIET_MODE" TEXT, "ACTIVITY_MODE" TEXT );
    CREATE TABLE "USER_WEIGHT_DATA" ( "USERNAME" TEXT NOT NULL, "DATE" TEXT NOT NULL, "WEIGHT" NUMERIC NOT NULL );
    CREATE TABLE "VOICE_COMMAND_REPLACE" ( "TEXT" TEXT NOT NULL UNIQUE, "REPLACE_WITH" TEXT NOT NULL, "USERNAME" TEXT NOT NULL );
    CREATE TABLE "VOICE_COMMAND_USER" ( "USERNAME" TEXT NOT NULL, "CONTEXT_TIME" INTEGER, "CONTEXT_KEY" TEXT, "CONTEXT_DATA" TEXT, PRIMARY KEY("USERNAME") );
    CREATE TABLE "WAKE_ON_LAN" ( "DEVICE" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "LOCATION" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device", "MAC_ADDRESS" TEXT NOT NULL );
    CREATE TABLE "XBOX_ONE_WOL" ( "ID" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, "NAME" TEXT NOT NULL, "LOCATION" TEXT NOT NULL, "IP_ADDRESS" TEXT NOT NULL, "XBOX_LIVE_ID" TEXT NOT NULL, "ICON" TEXT NOT NULL DEFAULT "device" );
    CREATE TABLE "ZWAVE_DIMMER" ( "NAME" TEXT, "LOCATION" TEXT, "ID" TEXT NOT NULL UNIQUE, "ICON" TEXT NOT NULL DEFAULT "device", "VALUE" TEXT NOT NULL DEFAULT "N/A", PRIMARY KEY("ID") );
    CREATE TABLE "ZWAVE_POWER_METER" ( "ROOM_ID" TEXT, "DEVICE_ID" TEXT NOT NULL UNIQUE, "DEVICE_NAME" TEXT, "ICON" TEXT NOT NULL DEFAULT "device", "VALUE" TEXT NOT NULL DEFAULT "N/A", "IS_RESET_DAILY" INTEGER NOT NULL DEFAULT 0, "PREV_VALUE" INTEGER DEFAULT 0, PRIMARY KEY("DEVICE_ID") );
    CREATE TABLE "ZWAVE_SENSOREN" ( "RAUM" TEXT, "ID" TEXT NOT NULL UNIQUE, "SHORTFORM" TEXT, "ICON" TEXT NOT NULL DEFAULT "device", "SAVE_DATA" BOOLEAN NOT NULL DEFAULT "true", "SENSOR_TYPE" TEXT, "VALUE" TEXT DEFAULT "N/A", PRIMARY KEY("ID") );
    CREATE TABLE "ZWAVE_SWITCHES" ( "NAME" TEXT, "LOCATION" TEXT, "ID" TEXT NOT NULL UNIQUE, "ICON" TEXT NOT NULL DEFAULT "device", "VALUE" TEXT DEFAULT "N/A", PRIMARY KEY("ID") );
    CREATE TABLE "ZWAVE_THERMOSTATS" ( "RAUM" TEXT NOT NULL, "NAME" TEXT NOT NULL, "THERMOSTAT_ID" TEXT NOT NULL UNIQUE, "TEMPERATUR_ID" TEXT, "ICON" TEXT NOT NULL DEFAULT "heating", "VALUE" TEXT DEFAULT "N/A", PRIMARY KEY("THERMOSTAT_ID") );    
    """

    db = get_database_con()
    with db:
        cur = db.cursor()
        cur.executescript(command)